# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

# This wrapper is part of IMP,
# Copyright 2007-2021 IMP Inventors. All rights reserved.

from __future__ import print_function, division, absolute_import
from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

import _IMP_npc

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

class IMP_NPC_SwigPyIterator(object):
    r"""Proxy of C++ swig::IMP_NPC_SwigPyIterator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _IMP_npc.delete_IMP_NPC_SwigPyIterator

    def value(self):
        r"""value(IMP_NPC_SwigPyIterator self) -> PyObject *"""
        return _IMP_npc.IMP_NPC_SwigPyIterator_value(self)

    def incr(self, n=1):
        r"""incr(IMP_NPC_SwigPyIterator self, size_t n=1) -> IMP_NPC_SwigPyIterator"""
        return _IMP_npc.IMP_NPC_SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        r"""decr(IMP_NPC_SwigPyIterator self, size_t n=1) -> IMP_NPC_SwigPyIterator"""
        return _IMP_npc.IMP_NPC_SwigPyIterator_decr(self, n)

    def distance(self, x):
        r"""distance(IMP_NPC_SwigPyIterator self, IMP_NPC_SwigPyIterator x) -> ptrdiff_t"""
        return _IMP_npc.IMP_NPC_SwigPyIterator_distance(self, x)

    def equal(self, x):
        r"""equal(IMP_NPC_SwigPyIterator self, IMP_NPC_SwigPyIterator x) -> bool"""
        return _IMP_npc.IMP_NPC_SwigPyIterator_equal(self, x)

    def copy(self):
        r"""copy(IMP_NPC_SwigPyIterator self) -> IMP_NPC_SwigPyIterator"""
        return _IMP_npc.IMP_NPC_SwigPyIterator_copy(self)

    def next(self):
        r"""next(IMP_NPC_SwigPyIterator self) -> PyObject *"""
        return _IMP_npc.IMP_NPC_SwigPyIterator_next(self)

    def __next__(self):
        r"""__next__(IMP_NPC_SwigPyIterator self) -> PyObject *"""
        return _IMP_npc.IMP_NPC_SwigPyIterator___next__(self)

    def previous(self):
        r"""previous(IMP_NPC_SwigPyIterator self) -> PyObject *"""
        return _IMP_npc.IMP_NPC_SwigPyIterator_previous(self)

    def advance(self, n):
        r"""advance(IMP_NPC_SwigPyIterator self, ptrdiff_t n) -> IMP_NPC_SwigPyIterator"""
        return _IMP_npc.IMP_NPC_SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        r"""__eq__(IMP_NPC_SwigPyIterator self, IMP_NPC_SwigPyIterator x) -> bool"""
        return _IMP_npc.IMP_NPC_SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        r"""__ne__(IMP_NPC_SwigPyIterator self, IMP_NPC_SwigPyIterator x) -> bool"""
        return _IMP_npc.IMP_NPC_SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        r"""__iadd__(IMP_NPC_SwigPyIterator self, ptrdiff_t n) -> IMP_NPC_SwigPyIterator"""
        return _IMP_npc.IMP_NPC_SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        r"""__isub__(IMP_NPC_SwigPyIterator self, ptrdiff_t n) -> IMP_NPC_SwigPyIterator"""
        return _IMP_npc.IMP_NPC_SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        r"""__add__(IMP_NPC_SwigPyIterator self, ptrdiff_t n) -> IMP_NPC_SwigPyIterator"""
        return _IMP_npc.IMP_NPC_SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        r"""
        __sub__(IMP_NPC_SwigPyIterator self, ptrdiff_t n) -> IMP_NPC_SwigPyIterator
        __sub__(IMP_NPC_SwigPyIterator self, IMP_NPC_SwigPyIterator x) -> ptrdiff_t
        """
        return _IMP_npc.IMP_NPC_SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register IMP_NPC_SwigPyIterator in _IMP_npc:
_IMP_npc.IMP_NPC_SwigPyIterator_swigregister(IMP_NPC_SwigPyIterator)


_value_types=[]
_object_types=[]
_raii_types=[]
_plural_types=[]

IMP_DEBUG = _IMP_npc.IMP_DEBUG

IMP_RELEASE = _IMP_npc.IMP_RELEASE

IMP_SILENT = _IMP_npc.IMP_SILENT

IMP_PROGRESS = _IMP_npc.IMP_PROGRESS

IMP_TERSE = _IMP_npc.IMP_TERSE

IMP_VERBOSE = _IMP_npc.IMP_VERBOSE

IMP_MEMORY = _IMP_npc.IMP_MEMORY

IMP_NONE = _IMP_npc.IMP_NONE

IMP_USAGE = _IMP_npc.IMP_USAGE

IMP_INTERNAL = _IMP_npc.IMP_INTERNAL

IMP_KERNEL_HAS_LOG4CXX = _IMP_npc.IMP_KERNEL_HAS_LOG4CXX

IMP_COMPILER_HAS_AUTO = _IMP_npc.IMP_COMPILER_HAS_AUTO

IMP_COMPILER_HAS_DEBUG_VECTOR = _IMP_npc.IMP_COMPILER_HAS_DEBUG_VECTOR

IMP_COMPILER_HAS_UNIQUE_PTR = _IMP_npc.IMP_COMPILER_HAS_UNIQUE_PTR

IMP_KERNEL_HAS_BOOST_RANDOM = _IMP_npc.IMP_KERNEL_HAS_BOOST_RANDOM

IMP_KERNEL_HAS_NUMPY = _IMP_npc.IMP_KERNEL_HAS_NUMPY

IMP_KERNEL_HAS_GPERFTOOLS = _IMP_npc.IMP_KERNEL_HAS_GPERFTOOLS

IMP_KERNEL_HAS_TCMALLOC_HEAPCHECKER = _IMP_npc.IMP_KERNEL_HAS_TCMALLOC_HEAPCHECKER

IMP_KERNEL_HAS_TCMALLOC_HEAPPROFILER = _IMP_npc.IMP_KERNEL_HAS_TCMALLOC_HEAPPROFILER

IMPKERNEL_SHOW_WARNINGS = _IMP_npc.IMPKERNEL_SHOW_WARNINGS


import sys
class _DirectorObjects(object):
    """@internal Simple class to keep references to director objects
       to prevent premature deletion."""
    def __init__(self):
        self._objects = []
    def register(self, obj):
        """Take a reference to a director object; will only work for
           refcounted C++ classes"""
        if hasattr(obj, 'get_ref_count'):
            self._objects.append(obj)
    def cleanup(self):
        """Only drop our reference and allow cleanup by Python if no other
           Python references exist (we hold 3 references: one in self._objects,
           one in x, and one in the argument list for getrefcount) *and* no
           other C++ references exist (the Python object always holds one)"""
        objs = [x for x in self._objects if sys.getrefcount(x) > 3 \
                                         or x.get_ref_count() > 1]
# Do in two steps so the references are kept until the end of the
# function (deleting references may trigger a fresh call to this method)
        self._objects = objs
    def get_object_count(self):
        """Get number of director objects (useful for testing only)"""
        return len(self._objects)
_director_objects = _DirectorObjects()

class _ostream(object):
    r"""Proxy of C++ std::ostream class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def write(self, osa_buf):
        r"""write(_ostream self, char const * osa_buf)"""
        return _IMP_npc._ostream_write(self, osa_buf)

# Register _ostream in _IMP_npc:
_IMP_npc._ostream_swigregister(_ostream)

IMP_COMPILER_HAS_OVERRIDE = _IMP_npc.IMP_COMPILER_HAS_OVERRIDE

IMP_COMPILER_HAS_FINAL = _IMP_npc.IMP_COMPILER_HAS_FINAL

IMP_HAS_NOEXCEPT = _IMP_npc.IMP_HAS_NOEXCEPT

IMP_C_OPEN_BINARY = _IMP_npc.IMP_C_OPEN_BINARY

import IMP
IMP_CGAL_HAS_BOOST_FILESYSTEM = _IMP_npc.IMP_CGAL_HAS_BOOST_FILESYSTEM

IMP_CGAL_HAS_BOOST_PROGRAMOPTIONS = _IMP_npc.IMP_CGAL_HAS_BOOST_PROGRAMOPTIONS

IMP_CGAL_HAS_BOOST_RANDOM = _IMP_npc.IMP_CGAL_HAS_BOOST_RANDOM

IMP_CGAL_HAS_BOOST_SYSTEM = _IMP_npc.IMP_CGAL_HAS_BOOST_SYSTEM

IMP_CGAL_HAS_NUMPY = _IMP_npc.IMP_CGAL_HAS_NUMPY

IMPCGAL_SHOW_WARNINGS = _IMP_npc.IMPCGAL_SHOW_WARNINGS

import IMP.cgal
IMP_ALGEBRA_HAS_IMP_CGAL = _IMP_npc.IMP_ALGEBRA_HAS_IMP_CGAL

IMP_ALGEBRA_HAS_BOOST_FILESYSTEM = _IMP_npc.IMP_ALGEBRA_HAS_BOOST_FILESYSTEM

IMP_ALGEBRA_HAS_BOOST_PROGRAMOPTIONS = _IMP_npc.IMP_ALGEBRA_HAS_BOOST_PROGRAMOPTIONS

IMP_ALGEBRA_HAS_BOOST_RANDOM = _IMP_npc.IMP_ALGEBRA_HAS_BOOST_RANDOM

IMP_ALGEBRA_HAS_BOOST_SYSTEM = _IMP_npc.IMP_ALGEBRA_HAS_BOOST_SYSTEM

IMP_ALGEBRA_HAS_CGAL = _IMP_npc.IMP_ALGEBRA_HAS_CGAL

IMP_ALGEBRA_HAS_NUMPY = _IMP_npc.IMP_ALGEBRA_HAS_NUMPY

IMP_ALGEBRA_HAS_ANN = _IMP_npc.IMP_ALGEBRA_HAS_ANN

IMPALGEBRA_SHOW_WARNINGS = _IMP_npc.IMPALGEBRA_SHOW_WARNINGS

import IMP.algebra
IMP_DISPLAY_HAS_IMP_CGAL = _IMP_npc.IMP_DISPLAY_HAS_IMP_CGAL

IMP_DISPLAY_HAS_BOOST_FILESYSTEM = _IMP_npc.IMP_DISPLAY_HAS_BOOST_FILESYSTEM

IMP_DISPLAY_HAS_BOOST_PROGRAMOPTIONS = _IMP_npc.IMP_DISPLAY_HAS_BOOST_PROGRAMOPTIONS

IMP_DISPLAY_HAS_BOOST_RANDOM = _IMP_npc.IMP_DISPLAY_HAS_BOOST_RANDOM

IMP_DISPLAY_HAS_BOOST_SYSTEM = _IMP_npc.IMP_DISPLAY_HAS_BOOST_SYSTEM

IMP_DISPLAY_HAS_CGAL = _IMP_npc.IMP_DISPLAY_HAS_CGAL

IMP_DISPLAY_HAS_NUMPY = _IMP_npc.IMP_DISPLAY_HAS_NUMPY

IMPDISPLAY_SHOW_WARNINGS = _IMP_npc.IMPDISPLAY_SHOW_WARNINGS

import IMP.display
IMP_SCORE_FUNCTOR_HAS_IMP_CGAL = _IMP_npc.IMP_SCORE_FUNCTOR_HAS_IMP_CGAL

IMP_SCORE_FUNCTOR_HAS_BOOST_FILESYSTEM = _IMP_npc.IMP_SCORE_FUNCTOR_HAS_BOOST_FILESYSTEM

IMP_SCORE_FUNCTOR_HAS_BOOST_PROGRAMOPTIONS = _IMP_npc.IMP_SCORE_FUNCTOR_HAS_BOOST_PROGRAMOPTIONS

IMP_SCORE_FUNCTOR_HAS_BOOST_RANDOM = _IMP_npc.IMP_SCORE_FUNCTOR_HAS_BOOST_RANDOM

IMP_SCORE_FUNCTOR_HAS_BOOST_SYSTEM = _IMP_npc.IMP_SCORE_FUNCTOR_HAS_BOOST_SYSTEM

IMP_SCORE_FUNCTOR_HAS_CGAL = _IMP_npc.IMP_SCORE_FUNCTOR_HAS_CGAL

IMP_SCORE_FUNCTOR_HAS_HDF5 = _IMP_npc.IMP_SCORE_FUNCTOR_HAS_HDF5

IMP_SCORE_FUNCTOR_HAS_NUMPY = _IMP_npc.IMP_SCORE_FUNCTOR_HAS_NUMPY

IMPSCOREFUNCTOR_SHOW_WARNINGS = _IMP_npc.IMPSCOREFUNCTOR_SHOW_WARNINGS

import IMP.score_functor
IMP_CORE_HAS_IMP_CGAL = _IMP_npc.IMP_CORE_HAS_IMP_CGAL

IMP_CORE_HAS_IMP_KERNEL = _IMP_npc.IMP_CORE_HAS_IMP_KERNEL

IMP_CORE_HAS_BOOST_FILESYSTEM = _IMP_npc.IMP_CORE_HAS_BOOST_FILESYSTEM

IMP_CORE_HAS_BOOST_PROGRAMOPTIONS = _IMP_npc.IMP_CORE_HAS_BOOST_PROGRAMOPTIONS

IMP_CORE_HAS_BOOST_RANDOM = _IMP_npc.IMP_CORE_HAS_BOOST_RANDOM

IMP_CORE_HAS_BOOST_SYSTEM = _IMP_npc.IMP_CORE_HAS_BOOST_SYSTEM

IMP_CORE_HAS_CGAL = _IMP_npc.IMP_CORE_HAS_CGAL

IMP_CORE_HAS_HDF5 = _IMP_npc.IMP_CORE_HAS_HDF5

IMP_CORE_HAS_NUMPY = _IMP_npc.IMP_CORE_HAS_NUMPY

IMPCORE_SHOW_WARNINGS = _IMP_npc.IMPCORE_SHOW_WARNINGS

import IMP.core
IMP_NPC_HAS_IMP_ALGEBRA = _IMP_npc.IMP_NPC_HAS_IMP_ALGEBRA

IMP_NPC_HAS_IMP_CGAL = _IMP_npc.IMP_NPC_HAS_IMP_CGAL

IMP_NPC_HAS_IMP_DISPLAY = _IMP_npc.IMP_NPC_HAS_IMP_DISPLAY

IMP_NPC_HAS_IMP_KERNEL = _IMP_npc.IMP_NPC_HAS_IMP_KERNEL

IMP_NPC_HAS_IMP_SCORE_FUNCTOR = _IMP_npc.IMP_NPC_HAS_IMP_SCORE_FUNCTOR

IMP_NPC_HAS_BOOST_FILESYSTEM = _IMP_npc.IMP_NPC_HAS_BOOST_FILESYSTEM

IMP_NPC_HAS_BOOST_PROGRAMOPTIONS = _IMP_npc.IMP_NPC_HAS_BOOST_PROGRAMOPTIONS

IMP_NPC_HAS_BOOST_RANDOM = _IMP_npc.IMP_NPC_HAS_BOOST_RANDOM

IMP_NPC_HAS_BOOST_SYSTEM = _IMP_npc.IMP_NPC_HAS_BOOST_SYSTEM

IMP_NPC_HAS_CGAL = _IMP_npc.IMP_NPC_HAS_CGAL

IMP_NPC_HAS_HDF5 = _IMP_npc.IMP_NPC_HAS_HDF5

IMP_NPC_HAS_NUMPY = _IMP_npc.IMP_NPC_HAS_NUMPY

IMP_NPC_HAS_PYTHON_IHM = _IMP_npc.IMP_NPC_HAS_PYTHON_IHM

IMPNPC_SHOW_WARNINGS = _IMP_npc.IMPNPC_SHOW_WARNINGS


_object_types.append("ZAxialPositionLowerRestraint")


def _object_cast_to_ZAxialPositionLowerRestraint(o):
    r"""_object_cast_to_ZAxialPositionLowerRestraint(Object o) -> ZAxialPositionLowerRestraint"""
    return _IMP_npc._object_cast_to_ZAxialPositionLowerRestraint(o)

_object_types.append("ZAxialPositionUpperRestraint")


def _object_cast_to_ZAxialPositionUpperRestraint(o):
    r"""_object_cast_to_ZAxialPositionUpperRestraint(Object o) -> ZAxialPositionUpperRestraint"""
    return _IMP_npc._object_cast_to_ZAxialPositionUpperRestraint(o)

_object_types.append("XYRadialPositionLowerRestraint")


def _object_cast_to_XYRadialPositionLowerRestraint(o):
    r"""_object_cast_to_XYRadialPositionLowerRestraint(Object o) -> XYRadialPositionLowerRestraint"""
    return _IMP_npc._object_cast_to_XYRadialPositionLowerRestraint(o)

_object_types.append("XYRadialPositionUpperRestraint")


def _object_cast_to_XYRadialPositionUpperRestraint(o):
    r"""_object_cast_to_XYRadialPositionUpperRestraint(Object o) -> XYRadialPositionUpperRestraint"""
    return _IMP_npc._object_cast_to_XYRadialPositionUpperRestraint(o)

_object_types.append("ProteinContactRestraint")


def _object_cast_to_ProteinContactRestraint(o):
    r"""_object_cast_to_ProteinContactRestraint(Object o) -> ProteinContactRestraint"""
    return _IMP_npc._object_cast_to_ProteinContactRestraint(o)

_object_types.append("ProteinChainRestraint")


def _object_cast_to_ProteinChainRestraint(o):
    r"""_object_cast_to_ProteinChainRestraint(Object o) -> ProteinChainRestraint"""
    return _IMP_npc._object_cast_to_ProteinChainRestraint(o)

_object_types.append("MembraneSurfaceLocationRestraint")


def _object_cast_to_MembraneSurfaceLocationRestraint(o):
    r"""_object_cast_to_MembraneSurfaceLocationRestraint(Object o) -> MembraneSurfaceLocationRestraint"""
    return _IMP_npc._object_cast_to_MembraneSurfaceLocationRestraint(o)

_object_types.append("PoreSideVolumeLocationRestraint")


def _object_cast_to_PoreSideVolumeLocationRestraint(o):
    r"""_object_cast_to_PoreSideVolumeLocationRestraint(Object o) -> PoreSideVolumeLocationRestraint"""
    return _IMP_npc._object_cast_to_PoreSideVolumeLocationRestraint(o)

_object_types.append("PerinuclearVolumeLocationRestraint")


def _object_cast_to_PerinuclearVolumeLocationRestraint(o):
    r"""_object_cast_to_PerinuclearVolumeLocationRestraint(Object o) -> PerinuclearVolumeLocationRestraint"""
    return _IMP_npc._object_cast_to_PerinuclearVolumeLocationRestraint(o)

_object_types.append("AssemblySymmetryByDistanceRestraint")


def _object_cast_to_AssemblySymmetryByDistanceRestraint(o):
    r"""_object_cast_to_AssemblySymmetryByDistanceRestraint(Object o) -> AssemblySymmetryByDistanceRestraint"""
    return _IMP_npc._object_cast_to_AssemblySymmetryByDistanceRestraint(o)

_object_types.append("AssemblySymmetryByDihedralRestraint")


def _object_cast_to_AssemblySymmetryByDihedralRestraint(o):
    r"""_object_cast_to_AssemblySymmetryByDihedralRestraint(Object o) -> AssemblySymmetryByDihedralRestraint"""
    return _IMP_npc._object_cast_to_AssemblySymmetryByDihedralRestraint(o)

_object_types.append("ProteinProximityRestraint")


def _object_cast_to_ProteinProximityRestraint(o):
    r"""_object_cast_to_ProteinProximityRestraint(Object o) -> ProteinProximityRestraint"""
    return _IMP_npc._object_cast_to_ProteinProximityRestraint(o)

_object_types.append("CompositeRestraint")


def _object_cast_to_CompositeRestraint(o):
    r"""_object_cast_to_CompositeRestraint(Object o) -> CompositeRestraint"""
    return _IMP_npc._object_cast_to_CompositeRestraint(o)

_object_types.append("MinimumSphereDistancePairScore")


def _object_cast_to_MinimumSphereDistancePairScore(o):
    r"""_object_cast_to_MinimumSphereDistancePairScore(Object o) -> MinimumSphereDistancePairScore"""
    return _IMP_npc._object_cast_to_MinimumSphereDistancePairScore(o)

_object_types.append("MembraneExclusionRestraint")


def _object_cast_to_MembraneExclusionRestraint(o):
    r"""_object_cast_to_MembraneExclusionRestraint(Object o) -> MembraneExclusionRestraint"""
    return _IMP_npc._object_cast_to_MembraneExclusionRestraint(o)

_object_types.append("MembraneSurfaceLocationConditionalRestraint")


def _object_cast_to_MembraneSurfaceLocationConditionalRestraint(o):
    r"""_object_cast_to_MembraneSurfaceLocationConditionalRestraint(Object o) -> MembraneSurfaceLocationConditionalRestraint"""
    return _IMP_npc._object_cast_to_MembraneSurfaceLocationConditionalRestraint(o)

_object_types.append("XYRadialPositionRestraint")


def _object_cast_to_XYRadialPositionRestraint(o):
    r"""_object_cast_to_XYRadialPositionRestraint(Object o) -> XYRadialPositionRestraint"""
    return _IMP_npc._object_cast_to_XYRadialPositionRestraint(o)

_object_types.append("YAxialPositionLowerRestraint")


def _object_cast_to_YAxialPositionLowerRestraint(o):
    r"""_object_cast_to_YAxialPositionLowerRestraint(Object o) -> YAxialPositionLowerRestraint"""
    return _IMP_npc._object_cast_to_YAxialPositionLowerRestraint(o)

_object_types.append("YAxialPositionRestraint")


def _object_cast_to_YAxialPositionRestraint(o):
    r"""_object_cast_to_YAxialPositionRestraint(Object o) -> YAxialPositionRestraint"""
    return _IMP_npc._object_cast_to_YAxialPositionRestraint(o)

_object_types.append("YAxialPositionUpperRestraint")


def _object_cast_to_YAxialPositionUpperRestraint(o):
    r"""_object_cast_to_YAxialPositionUpperRestraint(Object o) -> YAxialPositionUpperRestraint"""
    return _IMP_npc._object_cast_to_YAxialPositionUpperRestraint(o)

_object_types.append("ZAxialPositionRestraint")


def _object_cast_to_ZAxialPositionRestraint(o):
    r"""_object_cast_to_ZAxialPositionRestraint(Object o) -> ZAxialPositionRestraint"""
    return _IMP_npc._object_cast_to_ZAxialPositionRestraint(o)
class ZAxialPositionRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::ZAxialPositionRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(ZAxialPositionRestraint self, Model m, _SingletonContainerAdaptor sc, double lower_bound, double upper_bound, bool consider_radius, double sigma=1) -> ZAxialPositionRestraint
        __init__(ZAxialPositionRestraint self, Model m, double lower_bound, double upper_bound, bool consider_radius, double sigma=1) -> ZAxialPositionRestraint
        """
        _IMP_npc.ZAxialPositionRestraint_swiginit(self, _IMP_npc.new_ZAxialPositionRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(ZAxialPositionRestraint self, Particle p)"""
        return _IMP_npc.ZAxialPositionRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(ZAxialPositionRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.ZAxialPositionRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(ZAxialPositionRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.ZAxialPositionRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(ZAxialPositionRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.ZAxialPositionRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(ZAxialPositionRestraint self) -> VersionInfo"""
        return _IMP_npc.ZAxialPositionRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(ZAxialPositionRestraint self) -> std::string"""
        return _IMP_npc.ZAxialPositionRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(ZAxialPositionRestraint self) -> std::string"""
        return _IMP_npc.ZAxialPositionRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_ZAxialPositionRestraint(o)


# Register ZAxialPositionRestraint in _IMP_npc:
_IMP_npc.ZAxialPositionRestraint_swigregister(ZAxialPositionRestraint)

class ZAxialPositionLowerRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::ZAxialPositionLowerRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(ZAxialPositionLowerRestraint self, Model m, _SingletonContainerAdaptor sc, double lower_bound, bool consider_radius, double sigma=1) -> ZAxialPositionLowerRestraint
        __init__(ZAxialPositionLowerRestraint self, Model m, double lower_bound, bool consider_radius, double sigma=1) -> ZAxialPositionLowerRestraint
        """
        _IMP_npc.ZAxialPositionLowerRestraint_swiginit(self, _IMP_npc.new_ZAxialPositionLowerRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(ZAxialPositionLowerRestraint self, Particle p)"""
        return _IMP_npc.ZAxialPositionLowerRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(ZAxialPositionLowerRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.ZAxialPositionLowerRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(ZAxialPositionLowerRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.ZAxialPositionLowerRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(ZAxialPositionLowerRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.ZAxialPositionLowerRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(ZAxialPositionLowerRestraint self) -> VersionInfo"""
        return _IMP_npc.ZAxialPositionLowerRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(ZAxialPositionLowerRestraint self) -> std::string"""
        return _IMP_npc.ZAxialPositionLowerRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(ZAxialPositionLowerRestraint self) -> std::string"""
        return _IMP_npc.ZAxialPositionLowerRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_ZAxialPositionLowerRestraint(o)


# Register ZAxialPositionLowerRestraint in _IMP_npc:
_IMP_npc.ZAxialPositionLowerRestraint_swigregister(ZAxialPositionLowerRestraint)

class ZAxialPositionUpperRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::ZAxialPositionUpperRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(ZAxialPositionUpperRestraint self, Model m, _SingletonContainerAdaptor sc, double upper_bound, bool consider_radius, double sigma=1) -> ZAxialPositionUpperRestraint
        __init__(ZAxialPositionUpperRestraint self, Model m, double upper_bound, bool consider_radius, double sigma=1) -> ZAxialPositionUpperRestraint
        """
        _IMP_npc.ZAxialPositionUpperRestraint_swiginit(self, _IMP_npc.new_ZAxialPositionUpperRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(ZAxialPositionUpperRestraint self, Particle p)"""
        return _IMP_npc.ZAxialPositionUpperRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(ZAxialPositionUpperRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.ZAxialPositionUpperRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(ZAxialPositionUpperRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.ZAxialPositionUpperRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(ZAxialPositionUpperRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.ZAxialPositionUpperRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(ZAxialPositionUpperRestraint self) -> VersionInfo"""
        return _IMP_npc.ZAxialPositionUpperRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(ZAxialPositionUpperRestraint self) -> std::string"""
        return _IMP_npc.ZAxialPositionUpperRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(ZAxialPositionUpperRestraint self) -> std::string"""
        return _IMP_npc.ZAxialPositionUpperRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_ZAxialPositionUpperRestraint(o)


# Register ZAxialPositionUpperRestraint in _IMP_npc:
_IMP_npc.ZAxialPositionUpperRestraint_swigregister(ZAxialPositionUpperRestraint)

class YAxialPositionRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::YAxialPositionRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(YAxialPositionRestraint self, Model m, _SingletonContainerAdaptor sc, double lower_bound, double upper_bound, bool consider_radius, double sigma=1) -> YAxialPositionRestraint
        __init__(YAxialPositionRestraint self, Model m, double lower_bound, double upper_bound, bool consider_radius, double sigma=1) -> YAxialPositionRestraint
        """
        _IMP_npc.YAxialPositionRestraint_swiginit(self, _IMP_npc.new_YAxialPositionRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(YAxialPositionRestraint self, Particle p)"""
        return _IMP_npc.YAxialPositionRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(YAxialPositionRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.YAxialPositionRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(YAxialPositionRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.YAxialPositionRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(YAxialPositionRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.YAxialPositionRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(YAxialPositionRestraint self) -> VersionInfo"""
        return _IMP_npc.YAxialPositionRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(YAxialPositionRestraint self) -> std::string"""
        return _IMP_npc.YAxialPositionRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(YAxialPositionRestraint self) -> std::string"""
        return _IMP_npc.YAxialPositionRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_YAxialPositionRestraint(o)


# Register YAxialPositionRestraint in _IMP_npc:
_IMP_npc.YAxialPositionRestraint_swigregister(YAxialPositionRestraint)

class YAxialPositionLowerRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::YAxialPositionLowerRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(YAxialPositionLowerRestraint self, Model m, _SingletonContainerAdaptor sc, double lower_bound, bool consider_radius, double sigma=1) -> YAxialPositionLowerRestraint
        __init__(YAxialPositionLowerRestraint self, Model m, double lower_bound, bool consider_radius, double sigma=1) -> YAxialPositionLowerRestraint
        """
        _IMP_npc.YAxialPositionLowerRestraint_swiginit(self, _IMP_npc.new_YAxialPositionLowerRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(YAxialPositionLowerRestraint self, Particle p)"""
        return _IMP_npc.YAxialPositionLowerRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(YAxialPositionLowerRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.YAxialPositionLowerRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(YAxialPositionLowerRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.YAxialPositionLowerRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(YAxialPositionLowerRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.YAxialPositionLowerRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(YAxialPositionLowerRestraint self) -> VersionInfo"""
        return _IMP_npc.YAxialPositionLowerRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(YAxialPositionLowerRestraint self) -> std::string"""
        return _IMP_npc.YAxialPositionLowerRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(YAxialPositionLowerRestraint self) -> std::string"""
        return _IMP_npc.YAxialPositionLowerRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_YAxialPositionLowerRestraint(o)


# Register YAxialPositionLowerRestraint in _IMP_npc:
_IMP_npc.YAxialPositionLowerRestraint_swigregister(YAxialPositionLowerRestraint)

class YAxialPositionUpperRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::YAxialPositionUpperRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(YAxialPositionUpperRestraint self, Model m, _SingletonContainerAdaptor sc, double upper_bound, bool consider_radius, double sigma=1) -> YAxialPositionUpperRestraint
        __init__(YAxialPositionUpperRestraint self, Model m, double upper_bound, bool consider_radius, double sigma=1) -> YAxialPositionUpperRestraint
        """
        _IMP_npc.YAxialPositionUpperRestraint_swiginit(self, _IMP_npc.new_YAxialPositionUpperRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(YAxialPositionUpperRestraint self, Particle p)"""
        return _IMP_npc.YAxialPositionUpperRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(YAxialPositionUpperRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.YAxialPositionUpperRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(YAxialPositionUpperRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.YAxialPositionUpperRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(YAxialPositionUpperRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.YAxialPositionUpperRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(YAxialPositionUpperRestraint self) -> VersionInfo"""
        return _IMP_npc.YAxialPositionUpperRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(YAxialPositionUpperRestraint self) -> std::string"""
        return _IMP_npc.YAxialPositionUpperRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(YAxialPositionUpperRestraint self) -> std::string"""
        return _IMP_npc.YAxialPositionUpperRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_YAxialPositionUpperRestraint(o)


# Register YAxialPositionUpperRestraint in _IMP_npc:
_IMP_npc.YAxialPositionUpperRestraint_swigregister(YAxialPositionUpperRestraint)

class XYRadialPositionRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::XYRadialPositionRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(XYRadialPositionRestraint self, Model m, _SingletonContainerAdaptor sc, double lower_bound, double upper_bound, bool consider_radius, double sigma=1) -> XYRadialPositionRestraint
        __init__(XYRadialPositionRestraint self, Model m, double lower_bound, double upper_bound, bool consider_radius, double sigma=1) -> XYRadialPositionRestraint
        """
        _IMP_npc.XYRadialPositionRestraint_swiginit(self, _IMP_npc.new_XYRadialPositionRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(XYRadialPositionRestraint self, Particle p)"""
        return _IMP_npc.XYRadialPositionRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(XYRadialPositionRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.XYRadialPositionRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(XYRadialPositionRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.XYRadialPositionRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(XYRadialPositionRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.XYRadialPositionRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(XYRadialPositionRestraint self) -> VersionInfo"""
        return _IMP_npc.XYRadialPositionRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(XYRadialPositionRestraint self) -> std::string"""
        return _IMP_npc.XYRadialPositionRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(XYRadialPositionRestraint self) -> std::string"""
        return _IMP_npc.XYRadialPositionRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_XYRadialPositionRestraint(o)


# Register XYRadialPositionRestraint in _IMP_npc:
_IMP_npc.XYRadialPositionRestraint_swigregister(XYRadialPositionRestraint)

class XYRadialPositionLowerRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::XYRadialPositionLowerRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(XYRadialPositionLowerRestraint self, Model m, _SingletonContainerAdaptor sc, double lower_bound, bool consider_radius, double sigma=1) -> XYRadialPositionLowerRestraint
        __init__(XYRadialPositionLowerRestraint self, Model m, double lower_bound, bool consider_radius, double sigma=1) -> XYRadialPositionLowerRestraint
        """
        _IMP_npc.XYRadialPositionLowerRestraint_swiginit(self, _IMP_npc.new_XYRadialPositionLowerRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(XYRadialPositionLowerRestraint self, Particle p)"""
        return _IMP_npc.XYRadialPositionLowerRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(XYRadialPositionLowerRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.XYRadialPositionLowerRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(XYRadialPositionLowerRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.XYRadialPositionLowerRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(XYRadialPositionLowerRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.XYRadialPositionLowerRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(XYRadialPositionLowerRestraint self) -> VersionInfo"""
        return _IMP_npc.XYRadialPositionLowerRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(XYRadialPositionLowerRestraint self) -> std::string"""
        return _IMP_npc.XYRadialPositionLowerRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(XYRadialPositionLowerRestraint self) -> std::string"""
        return _IMP_npc.XYRadialPositionLowerRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_XYRadialPositionLowerRestraint(o)


# Register XYRadialPositionLowerRestraint in _IMP_npc:
_IMP_npc.XYRadialPositionLowerRestraint_swigregister(XYRadialPositionLowerRestraint)

class XYRadialPositionUpperRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::XYRadialPositionUpperRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(XYRadialPositionUpperRestraint self, Model m, _SingletonContainerAdaptor sc, double upper_bound, bool consider_radius, double sigma=1) -> XYRadialPositionUpperRestraint
        __init__(XYRadialPositionUpperRestraint self, Model m, double upper_bound, bool consider_radius, double sigma=1) -> XYRadialPositionUpperRestraint
        """
        _IMP_npc.XYRadialPositionUpperRestraint_swiginit(self, _IMP_npc.new_XYRadialPositionUpperRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(XYRadialPositionUpperRestraint self, Particle p)"""
        return _IMP_npc.XYRadialPositionUpperRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(XYRadialPositionUpperRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.XYRadialPositionUpperRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(XYRadialPositionUpperRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.XYRadialPositionUpperRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(XYRadialPositionUpperRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.XYRadialPositionUpperRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(XYRadialPositionUpperRestraint self) -> VersionInfo"""
        return _IMP_npc.XYRadialPositionUpperRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(XYRadialPositionUpperRestraint self) -> std::string"""
        return _IMP_npc.XYRadialPositionUpperRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(XYRadialPositionUpperRestraint self) -> std::string"""
        return _IMP_npc.XYRadialPositionUpperRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_XYRadialPositionUpperRestraint(o)


# Register XYRadialPositionUpperRestraint in _IMP_npc:
_IMP_npc.XYRadialPositionUpperRestraint_swigregister(XYRadialPositionUpperRestraint)

class ProteinContactRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::ProteinContactRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(ProteinContactRestraint self, Model m, _SingletonContainerAdaptor sc, double tolerance_factor, double sigma=0.1) -> ProteinContactRestraint
        __init__(ProteinContactRestraint self, Model m, double tolerance_factor, double sigma=0.1) -> ProteinContactRestraint
        """
        _IMP_npc.ProteinContactRestraint_swiginit(self, _IMP_npc.new_ProteinContactRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(ProteinContactRestraint self, Particle p)"""
        return _IMP_npc.ProteinContactRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(ProteinContactRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.ProteinContactRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(ProteinContactRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.ProteinContactRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(ProteinContactRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.ProteinContactRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(ProteinContactRestraint self) -> VersionInfo"""
        return _IMP_npc.ProteinContactRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(ProteinContactRestraint self) -> std::string"""
        return _IMP_npc.ProteinContactRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(ProteinContactRestraint self) -> std::string"""
        return _IMP_npc.ProteinContactRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_ProteinContactRestraint(o)


# Register ProteinContactRestraint in _IMP_npc:
_IMP_npc.ProteinContactRestraint_swigregister(ProteinContactRestraint)

class ProteinChainRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::ProteinChainRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(ProteinChainRestraint self, Model m, _SingletonContainerAdaptor sc, double sigma=0.1) -> ProteinChainRestraint
        __init__(ProteinChainRestraint self, Model m, double sigma=0.1) -> ProteinChainRestraint
        """
        _IMP_npc.ProteinChainRestraint_swiginit(self, _IMP_npc.new_ProteinChainRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(ProteinChainRestraint self, Particle p)"""
        return _IMP_npc.ProteinChainRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(ProteinChainRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.ProteinChainRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(ProteinChainRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.ProteinChainRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(ProteinChainRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.ProteinChainRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(ProteinChainRestraint self) -> VersionInfo"""
        return _IMP_npc.ProteinChainRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(ProteinChainRestraint self) -> std::string"""
        return _IMP_npc.ProteinChainRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(ProteinChainRestraint self) -> std::string"""
        return _IMP_npc.ProteinChainRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_ProteinChainRestraint(o)


# Register ProteinChainRestraint in _IMP_npc:
_IMP_npc.ProteinChainRestraint_swigregister(ProteinChainRestraint)

class MembraneSurfaceLocationRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::MembraneSurfaceLocationRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(MembraneSurfaceLocationRestraint self, Model m, _SingletonContainerAdaptor sc, double R, double r, double thickness, double sigma=2) -> MembraneSurfaceLocationRestraint
        __init__(MembraneSurfaceLocationRestraint self, Model m, double R, double r, double thickness, double sigma=2) -> MembraneSurfaceLocationRestraint
        """
        _IMP_npc.MembraneSurfaceLocationRestraint_swiginit(self, _IMP_npc.new_MembraneSurfaceLocationRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(MembraneSurfaceLocationRestraint self, Particle p)"""
        return _IMP_npc.MembraneSurfaceLocationRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(MembraneSurfaceLocationRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.MembraneSurfaceLocationRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(MembraneSurfaceLocationRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.MembraneSurfaceLocationRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(MembraneSurfaceLocationRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.MembraneSurfaceLocationRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(MembraneSurfaceLocationRestraint self) -> VersionInfo"""
        return _IMP_npc.MembraneSurfaceLocationRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(MembraneSurfaceLocationRestraint self) -> std::string"""
        return _IMP_npc.MembraneSurfaceLocationRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(MembraneSurfaceLocationRestraint self) -> std::string"""
        return _IMP_npc.MembraneSurfaceLocationRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_MembraneSurfaceLocationRestraint(o)


# Register MembraneSurfaceLocationRestraint in _IMP_npc:
_IMP_npc.MembraneSurfaceLocationRestraint_swigregister(MembraneSurfaceLocationRestraint)

class MembraneSurfaceLocationConditionalRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::MembraneSurfaceLocationConditionalRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(MembraneSurfaceLocationConditionalRestraint self, Model m, _SingletonContainerAdaptor sc1, _SingletonContainerAdaptor sc2, double R, double r, double thickness, double sigma=2) -> MembraneSurfaceLocationConditionalRestraint
        __init__(MembraneSurfaceLocationConditionalRestraint self, Model m, double R, double r, double thickness, double sigma=2) -> MembraneSurfaceLocationConditionalRestraint
        """
        _IMP_npc.MembraneSurfaceLocationConditionalRestraint_swiginit(self, _IMP_npc.new_MembraneSurfaceLocationConditionalRestraint(*args))

    def add_particle1(self, p):
        r"""add_particle1(MembraneSurfaceLocationConditionalRestraint self, Particle p)"""
        return _IMP_npc.MembraneSurfaceLocationConditionalRestraint_add_particle1(self, p)

    def add_particle2(self, p):
        r"""add_particle2(MembraneSurfaceLocationConditionalRestraint self, Particle p)"""
        return _IMP_npc.MembraneSurfaceLocationConditionalRestraint_add_particle2(self, p)

    def add_particles1(self, ps):
        r"""add_particles1(MembraneSurfaceLocationConditionalRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.MembraneSurfaceLocationConditionalRestraint_add_particles1(self, ps)

    def add_particles2(self, ps):
        r"""add_particles2(MembraneSurfaceLocationConditionalRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.MembraneSurfaceLocationConditionalRestraint_add_particles2(self, ps)

    def set_particles1(self, ps):
        r"""set_particles1(MembraneSurfaceLocationConditionalRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.MembraneSurfaceLocationConditionalRestraint_set_particles1(self, ps)

    def set_particles2(self, ps):
        r"""set_particles2(MembraneSurfaceLocationConditionalRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.MembraneSurfaceLocationConditionalRestraint_set_particles2(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(MembraneSurfaceLocationConditionalRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.MembraneSurfaceLocationConditionalRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(MembraneSurfaceLocationConditionalRestraint self) -> VersionInfo"""
        return _IMP_npc.MembraneSurfaceLocationConditionalRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(MembraneSurfaceLocationConditionalRestraint self) -> std::string"""
        return _IMP_npc.MembraneSurfaceLocationConditionalRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(MembraneSurfaceLocationConditionalRestraint self) -> std::string"""
        return _IMP_npc.MembraneSurfaceLocationConditionalRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_MembraneSurfaceLocationConditionalRestraint(o)


# Register MembraneSurfaceLocationConditionalRestraint in _IMP_npc:
_IMP_npc.MembraneSurfaceLocationConditionalRestraint_swigregister(MembraneSurfaceLocationConditionalRestraint)

class MembraneExclusionRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::MembraneExclusionRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(MembraneExclusionRestraint self, Model m, _SingletonContainerAdaptor sc, double R, double r, double thickness, double sigma=2) -> MembraneExclusionRestraint
        __init__(MembraneExclusionRestraint self, Model m, double R, double r, double thickness, double sigma=2) -> MembraneExclusionRestraint
        """
        _IMP_npc.MembraneExclusionRestraint_swiginit(self, _IMP_npc.new_MembraneExclusionRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(MembraneExclusionRestraint self, Particle p)"""
        return _IMP_npc.MembraneExclusionRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(MembraneExclusionRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.MembraneExclusionRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(MembraneExclusionRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.MembraneExclusionRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(MembraneExclusionRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.MembraneExclusionRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(MembraneExclusionRestraint self) -> VersionInfo"""
        return _IMP_npc.MembraneExclusionRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(MembraneExclusionRestraint self) -> std::string"""
        return _IMP_npc.MembraneExclusionRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(MembraneExclusionRestraint self) -> std::string"""
        return _IMP_npc.MembraneExclusionRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_MembraneExclusionRestraint(o)


# Register MembraneExclusionRestraint in _IMP_npc:
_IMP_npc.MembraneExclusionRestraint_swigregister(MembraneExclusionRestraint)

class PoreSideVolumeLocationRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::PoreSideVolumeLocationRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(PoreSideVolumeLocationRestraint self, Model m, _SingletonContainerAdaptor sc, double R, double r, double thickness, bool consider_radius, double sigma=2) -> PoreSideVolumeLocationRestraint
        __init__(PoreSideVolumeLocationRestraint self, Model m, double R, double r, double thickness, bool consider_radius, double sigma=2) -> PoreSideVolumeLocationRestraint
        """
        _IMP_npc.PoreSideVolumeLocationRestraint_swiginit(self, _IMP_npc.new_PoreSideVolumeLocationRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(PoreSideVolumeLocationRestraint self, Particle p)"""
        return _IMP_npc.PoreSideVolumeLocationRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(PoreSideVolumeLocationRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.PoreSideVolumeLocationRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(PoreSideVolumeLocationRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.PoreSideVolumeLocationRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(PoreSideVolumeLocationRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.PoreSideVolumeLocationRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(PoreSideVolumeLocationRestraint self) -> VersionInfo"""
        return _IMP_npc.PoreSideVolumeLocationRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(PoreSideVolumeLocationRestraint self) -> std::string"""
        return _IMP_npc.PoreSideVolumeLocationRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(PoreSideVolumeLocationRestraint self) -> std::string"""
        return _IMP_npc.PoreSideVolumeLocationRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_PoreSideVolumeLocationRestraint(o)


# Register PoreSideVolumeLocationRestraint in _IMP_npc:
_IMP_npc.PoreSideVolumeLocationRestraint_swigregister(PoreSideVolumeLocationRestraint)

class PerinuclearVolumeLocationRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::PerinuclearVolumeLocationRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(PerinuclearVolumeLocationRestraint self, Model m, _SingletonContainerAdaptor sc, double R, double r, double thickness, bool consider_radius, double sigma=2) -> PerinuclearVolumeLocationRestraint
        __init__(PerinuclearVolumeLocationRestraint self, Model m, double R, double r, double thickness, bool consider_radius, double sigma=2) -> PerinuclearVolumeLocationRestraint
        """
        _IMP_npc.PerinuclearVolumeLocationRestraint_swiginit(self, _IMP_npc.new_PerinuclearVolumeLocationRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(PerinuclearVolumeLocationRestraint self, Particle p)"""
        return _IMP_npc.PerinuclearVolumeLocationRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(PerinuclearVolumeLocationRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.PerinuclearVolumeLocationRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(PerinuclearVolumeLocationRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.PerinuclearVolumeLocationRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(PerinuclearVolumeLocationRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.PerinuclearVolumeLocationRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(PerinuclearVolumeLocationRestraint self) -> VersionInfo"""
        return _IMP_npc.PerinuclearVolumeLocationRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(PerinuclearVolumeLocationRestraint self) -> std::string"""
        return _IMP_npc.PerinuclearVolumeLocationRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(PerinuclearVolumeLocationRestraint self) -> std::string"""
        return _IMP_npc.PerinuclearVolumeLocationRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_PerinuclearVolumeLocationRestraint(o)


# Register PerinuclearVolumeLocationRestraint in _IMP_npc:
_IMP_npc.PerinuclearVolumeLocationRestraint_swigregister(PerinuclearVolumeLocationRestraint)

class AssemblySymmetryByDistanceRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::AssemblySymmetryByDistanceRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(AssemblySymmetryByDistanceRestraint self, Model m, _SingletonContainerAdaptor sc, double sigma=4) -> AssemblySymmetryByDistanceRestraint
        __init__(AssemblySymmetryByDistanceRestraint self, Model m, double sigma=4) -> AssemblySymmetryByDistanceRestraint
        """
        _IMP_npc.AssemblySymmetryByDistanceRestraint_swiginit(self, _IMP_npc.new_AssemblySymmetryByDistanceRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(AssemblySymmetryByDistanceRestraint self, Particle p)"""
        return _IMP_npc.AssemblySymmetryByDistanceRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(AssemblySymmetryByDistanceRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.AssemblySymmetryByDistanceRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(AssemblySymmetryByDistanceRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.AssemblySymmetryByDistanceRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(AssemblySymmetryByDistanceRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.AssemblySymmetryByDistanceRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(AssemblySymmetryByDistanceRestraint self) -> VersionInfo"""
        return _IMP_npc.AssemblySymmetryByDistanceRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(AssemblySymmetryByDistanceRestraint self) -> std::string"""
        return _IMP_npc.AssemblySymmetryByDistanceRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(AssemblySymmetryByDistanceRestraint self) -> std::string"""
        return _IMP_npc.AssemblySymmetryByDistanceRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_AssemblySymmetryByDistanceRestraint(o)


# Register AssemblySymmetryByDistanceRestraint in _IMP_npc:
_IMP_npc.AssemblySymmetryByDistanceRestraint_swigregister(AssemblySymmetryByDistanceRestraint)

class AssemblySymmetryByDihedralRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::AssemblySymmetryByDihedralRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(AssemblySymmetryByDihedralRestraint self, Model m, _SingletonContainerAdaptor sc, double sigma=0.1) -> AssemblySymmetryByDihedralRestraint
        __init__(AssemblySymmetryByDihedralRestraint self, Model m, double sigma=0.1) -> AssemblySymmetryByDihedralRestraint
        """
        _IMP_npc.AssemblySymmetryByDihedralRestraint_swiginit(self, _IMP_npc.new_AssemblySymmetryByDihedralRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(AssemblySymmetryByDihedralRestraint self, Particle p)"""
        return _IMP_npc.AssemblySymmetryByDihedralRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(AssemblySymmetryByDihedralRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.AssemblySymmetryByDihedralRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(AssemblySymmetryByDihedralRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.AssemblySymmetryByDihedralRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(AssemblySymmetryByDihedralRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.AssemblySymmetryByDihedralRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(AssemblySymmetryByDihedralRestraint self) -> VersionInfo"""
        return _IMP_npc.AssemblySymmetryByDihedralRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(AssemblySymmetryByDihedralRestraint self) -> std::string"""
        return _IMP_npc.AssemblySymmetryByDihedralRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(AssemblySymmetryByDihedralRestraint self) -> std::string"""
        return _IMP_npc.AssemblySymmetryByDihedralRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_AssemblySymmetryByDihedralRestraint(o)


# Register AssemblySymmetryByDihedralRestraint in _IMP_npc:
_IMP_npc.AssemblySymmetryByDihedralRestraint_swigregister(AssemblySymmetryByDihedralRestraint)

class ProteinProximityRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::ProteinProximityRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(ProteinProximityRestraint self, Model m, _SingletonContainerAdaptor sc, double max_dist, double sigma=0.1) -> ProteinProximityRestraint
        __init__(ProteinProximityRestraint self, Model m, double max_dist, double sigma=0.1) -> ProteinProximityRestraint
        """
        _IMP_npc.ProteinProximityRestraint_swiginit(self, _IMP_npc.new_ProteinProximityRestraint(*args))

    def add_particle(self, p):
        r"""add_particle(ProteinProximityRestraint self, Particle p)"""
        return _IMP_npc.ProteinProximityRestraint_add_particle(self, p)

    def add_particles(self, ps):
        r"""add_particles(ProteinProximityRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.ProteinProximityRestraint_add_particles(self, ps)

    def set_particles(self, ps):
        r"""set_particles(ProteinProximityRestraint self, IMP::ParticlesTemp const & ps)"""
        return _IMP_npc.ProteinProximityRestraint_set_particles(self, ps)

    def do_get_inputs(self):
        r"""do_get_inputs(ProteinProximityRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.ProteinProximityRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(ProteinProximityRestraint self) -> VersionInfo"""
        return _IMP_npc.ProteinProximityRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(ProteinProximityRestraint self) -> std::string"""
        return _IMP_npc.ProteinProximityRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(ProteinProximityRestraint self) -> std::string"""
        return _IMP_npc.ProteinProximityRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_ProteinProximityRestraint(o)


# Register ProteinProximityRestraint in _IMP_npc:
_IMP_npc.ProteinProximityRestraint_swigregister(ProteinProximityRestraint)

class CompositeRestraint(IMP.Restraint):
    r"""Proxy of C++ IMP::npc::CompositeRestraint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, m, ps):
        r"""__init__(CompositeRestraint self, Model m, PairScore ps) -> CompositeRestraint"""
        _IMP_npc.CompositeRestraint_swiginit(self, _IMP_npc.new_CompositeRestraint(m, ps))

    def add_type(self, ps):
        r"""add_type(CompositeRestraint self, IMP::ParticleIndexes ps)"""
        return _IMP_npc.CompositeRestraint_add_type(self, ps)

    def get_connected_pairs(self):
        r"""get_connected_pairs(CompositeRestraint self) -> IMP::ParticleIndexPairs"""
        return _IMP_npc.CompositeRestraint_get_connected_pairs(self)

    def do_get_inputs(self):
        r"""do_get_inputs(CompositeRestraint self) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.CompositeRestraint_do_get_inputs(self)

    def get_version_info(self):
        r"""get_version_info(CompositeRestraint self) -> VersionInfo"""
        return _IMP_npc.CompositeRestraint_get_version_info(self)

    def __str__(self):
        r"""__str__(CompositeRestraint self) -> std::string"""
        return _IMP_npc.CompositeRestraint___str__(self)

    def __repr__(self):
        r"""__repr__(CompositeRestraint self) -> std::string"""
        return _IMP_npc.CompositeRestraint___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_CompositeRestraint(o)


# Register CompositeRestraint in _IMP_npc:
_IMP_npc.CompositeRestraint_swigregister(CompositeRestraint)

class MinimumSphereDistancePairScore(IMP.PairScore):
    r"""Proxy of C++ IMP::npc::MinimumSphereDistancePairScore class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, f, transforms):
        r"""__init__(MinimumSphereDistancePairScore self, UnaryFunction f, IMP::algebra::Transformation3Ds transforms) -> MinimumSphereDistancePairScore"""
        _IMP_npc.MinimumSphereDistancePairScore_swiginit(self, _IMP_npc.new_MinimumSphereDistancePairScore(f, transforms))

    def do_get_inputs(self, m, pis):
        r"""do_get_inputs(MinimumSphereDistancePairScore self, Model m, IMP::ParticleIndexes const & pis) -> IMP::ModelObjectsTemp"""
        return _IMP_npc.MinimumSphereDistancePairScore_do_get_inputs(self, m, pis)

    def get_version_info(self):
        r"""get_version_info(MinimumSphereDistancePairScore self) -> VersionInfo"""
        return _IMP_npc.MinimumSphereDistancePairScore_get_version_info(self)

    def __str__(self):
        r"""__str__(MinimumSphereDistancePairScore self) -> std::string"""
        return _IMP_npc.MinimumSphereDistancePairScore___str__(self)

    def __repr__(self):
        r"""__repr__(MinimumSphereDistancePairScore self) -> std::string"""
        return _IMP_npc.MinimumSphereDistancePairScore___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_MinimumSphereDistancePairScore(o)


# Register MinimumSphereDistancePairScore in _IMP_npc:
_IMP_npc.MinimumSphereDistancePairScore_swigregister(MinimumSphereDistancePairScore)


def get_module_version():
    r"""get_module_version() -> std::string const"""
    return _IMP_npc.get_module_version()

def get_example_path(fname):
    r"""get_example_path(std::string fname) -> std::string"""
    return _IMP_npc.get_example_path(fname)

def get_data_path(fname):
    r"""get_data_path(std::string fname) -> std::string"""
    return _IMP_npc.get_data_path(fname)

from . import _version_check
_version_check.check_version(get_module_version())
__version__ = get_module_version()






## \example pmi/selection.py
"""This script demonstrates a few different ways you can perform selection in PMI.
In PMI we first set up molecules at various resolutions.
Then you call System.build() and this creates all requested representations.
"""

import IMP
import IMP.atom
import IMP.rmf
import IMP.pmi
import IMP.pmi.topology
import IMP.pmi.dof
import IMP.pmi.macros
import IMP.pmi.restraints
import IMP.pmi.restraints.stereochemistry
import sys

IMP.setup_from_argv(sys.argv, "Selection in PMI")

################ BEFORE BUILDING ###################
# Preliminaries: read sequences, create a system and a state
seqs = IMP.pmi.topology.Sequences(IMP.pmi.get_example_path('data/1WCM.fasta'))
mdl = IMP.Model()
s = IMP.pmi.topology.System(mdl)
st = s.create_state()

# Create a molecule. This sets up "TempResidues" for all elements in the sequence.
mol = st.create_molecule("Rpb4",seqs["1WCM:D"],chain_id="D")

# No structure has been built yet - you have to call 'Molecule.add_representation'
#  The first argument to that function is a set of "TempResidues"
#  Here are some ways of getting them:

# Slice a molecule (uses python-style 0-ordering)
myres1 = mol[0:10]

# Use PDB numbering with residue_range (inclusive on both ends)
myres2 = mol.residue_range('1','10')

# If you add a PDB, this returns the set of TempResidues that were in the PDB file
atomic = mol.add_structure(IMP.pmi.get_example_path('data/1WCM_fitted.pdb'),
                           chain_id="D")

# You can also call these helper functions at any time
atomic = mol.get_atomic_residues()
non_atomic = mol.get_non_atomic_residues()

# All of the above objects are OrderedSets, so you can actually perform set operations
myres3 = mol[0:50] - atomic # non-atomic residues from 0 to 50
myres4 = mol[0:50] & atomic # atomic residues between 0 and 50

# Finally, pass your favorite handle to add_representation
mol.add_representation(myres3,
                       resolutions=[1]) #creates beads with size 1 (non-atomic can only have one beadsize)
mol.add_representation(myres4,
                       resolutions=[1,10]) #creates beads with size 1 and 10

# When you have decided all representations, call build()
#  This returns an IMP hierarchy
hier = s.build()

# View your creation with this function
IMP.atom.show_with_representations(hier)


################ AFTER BUILDING ###################
# After building, only what you requested with add_representation() can be selected

# PMI selection
# Most PMI functions will let you pass Molecules or TempResidues and it will automatically gather things.
conn = IMP.pmi.restraints.stereochemistry.ConnectivityRestraint(mol,
                                                                resolution=1)

# Similarly for rigid body creation, we recommend passing PMI objects and it will gather all resolutions
dof = IMP.pmi.dof.DegreesOfFreedom(mdl)
dof.create_rigid_body(mol,
                      nonrigid_parts = non_atomic,
                      resolution='all')

# Advanced: IMP selection
# Note if you request a resolution this will find the NEAREST available resolution.
#  E.g. if only resolution 1 is built, those particles will be returned below:
sel = IMP.atom.Selection(hier,resolution=10,molecule="Rpb4",residue_indexes=range(1,10))
particles = sel.get_selected_particles()

# Retrieving the molecule object
# The molecules are stored within the state, so you can do :
all_mol_copies = st.molecules["Rpb4"] # a list of all copies
mol = all_mol_copies[0]



## \example em/local_fitting.py
# Shows how to locally refine a fit of a protein inside
# its density using a MC/CG optimization protocol.
# This example does not necessarily converge to the global minimum
# as that may require more optimization steps.
# If one wishes to use this example as a template for real refinement purposes,
# please adjust the parameters of the function IMP.em.local_rigid_fitting
# accordingly.

from __future__ import print_function
import IMP.em
import IMP.core
import IMP.atom
import random
import math
import sys

IMP.setup_from_argv(sys.argv, "local fitting")

IMP.set_log_level(IMP.SILENT)
IMP.set_check_level(IMP.NONE)
m = IMP.Model()
# 1. setup the input protein
# 1.1 select a selector.
# using NonWater selector is more accurate but slower
# sel=IMP.atom.NonWaterPDBSelector()
sel = IMP.atom.CAlphaPDBSelector()
# 1.2 read the protein
mh = IMP.atom.read_pdb(IMP.em.get_example_path("input.pdb"), m, sel)
mh_ref = IMP.atom.read_pdb(IMP.em.get_example_path("input.pdb"), m, sel)
# 1.3 add radius info to each atom, otherwise the resampling would fail.
IMP.atom.add_radii(mh)
IMP.atom.add_radii(mh_ref)
ps = IMP.core.get_leaves(mh)
ps_ref = IMP.core.get_leaves(mh_ref)
# 2. read the density map of the protein
resolution = 8.
voxel_size = 1.5
dmap = IMP.em.read_map(
    IMP.em.get_example_path("input.mrc"), IMP.em.MRCReaderWriter())
dmap.get_header_writable().set_resolution(resolution)
# 3. The protein is now fitted correctly in the density. We can validate
# that by making sure that the cross-correlation score is close to 1.

# 3.1 generate a sampled density map to the same resolution and spacing as
# the target density map. Note that the function we are going to use
# (cross_correlation_coefficient) expects to get the same map dimensions as
# the target density map.
sampled_input_density = IMP.em.SampledDensityMap(dmap.get_header())
sampled_input_density.set_particles(ps)
sampled_input_density.resample()
sampled_input_density.calcRMS()
IMP.em.write_map(sampled_input_density, "vv0.mrc", IMP.em.MRCReaderWriter())
# 3.2 calculate the cross-correlation score, which should be close to 1
best_score = IMP.em.get_coarse_cc_coefficient(
    dmap, sampled_input_density, sampled_input_density.get_header().dmin)
print("The CC score of the native transformation is:", best_score)

# 4. To demonstrate local fitting we locally rotate and translate the
# protein and show how we can go back to the correct placement.

# 4.1 define a local transformation
translation = IMP.algebra.get_random_vector_in(
    IMP.algebra.get_unit_bounding_box_3d())
axis = IMP.algebra.get_random_vector_on(IMP.algebra.get_unit_sphere_3d())
rand_angle = random.uniform(-70. / 180 * math.pi, 70. / 180 * math.pi)
r = IMP.algebra.get_rotation_about_axis(axis, rand_angle)
local_trans = IMP.algebra.Transformation3D(r, translation)
# 4.2 rotate the protein
# prot_xyz=IMP.core.XYZs(IMP.core.get_leaves(mh))
# for xyz in prot_xyz:
#     xyz.set_coordinates(local_trans.get_transformed(xyz.get_coordinates()))
# 4.2 set the protein as a rigid body
IMP.atom.create_rigid_body(mh)
prot_rb = IMP.core.RigidMember(IMP.core.get_leaves(mh)[0]).get_rigid_body()
# 4.3 apply the transformation to the protein
IMP.core.transform(prot_rb, local_trans)
m.update()  # to make sure the transformation was applied
# 4.4 print the new correlation score, should be lower than before
print(len(IMP.core.get_leaves(mh)))
IMP.atom.write_pdb(mh, "input2.pdb")
sampled_input_density.resample()
sampled_input_density.calcRMS()
IMP.em.write_map(sampled_input_density, "vv.mrc", IMP.em.MRCReaderWriter())
start_score = IMP.em.get_coarse_cc_coefficient(
    dmap, sampled_input_density, sampled_input_density.get_header().dmin)
start_rmsd = IMP.atom.get_rmsd(IMP.core.XYZs(ps), IMP.core.XYZs(ps_ref))
print("The start score is:", start_score, "with rmsd of:", start_rmsd)
# 5. apply local fitting
# 5.1 run local fitting
print("performing local refinement, may run for 3-4 minutes")
# translate the molecule to the center of the density
IMP.core.transform(prot_rb, IMP.algebra.Transformation3D(
    IMP.algebra.get_identity_rotation_3d(), dmap.get_centroid() - IMP.core.get_centroid(ps)))
m.update()  # to make sure the transformation was applied
sampled_input_density.resample()
sampled_input_density.calcRMS()
rmsd = IMP.atom.get_rmsd(IMP.core.XYZs(ps), IMP.core.XYZs(ps_ref))
score2 = IMP.em.get_coarse_cc_coefficient(
    dmap, sampled_input_density, sampled_input_density.get_header().dmin)
print("The score after centering is:", score2, "with rmsd of:", rmsd)
# IMP.em.local_rigid_fitting_grid_search(
#    ps,IMP.core.XYZR.get_radius_key(),
#    IMP.atom.Mass.get_mass_key(),
#    dmap,fitting_sols)

refiner = IMP.core.LeavesRefiner(IMP.atom.Hierarchy.get_traits())
fitting_sols = IMP.em.local_rigid_fitting(
    mh.get_particle(), refiner,
    IMP.atom.Mass.get_mass_key(),
    dmap, [], 2, 10, 10)

# 5.2 report best result
# 5.2.1 transform the protein to the preferred transformation
print("The start score is:", start_score, "with rmsd of:", start_rmsd)
for i in range(fitting_sols.get_number_of_solutions()):
    IMP.core.transform(prot_rb, fitting_sols.get_transformation(i))
    # prot_rb.set_reference_frame(IMP.algebra.ReferenceFrame3D(fitting_sols.get_transformation(i)))
    m.update()  # to make sure the transformation was applied
# 5.2.2 calc rmsd to native configuration
    rmsd = IMP.atom.get_rmsd(
        IMP.core.XYZs(ps), IMP.core.XYZs(IMP.core.get_leaves(mh_ref)))
    IMP.atom.write_pdb(mh, "temp_" + str(i) + ".pdb")
    print("Fit with index:", i, " with cc: ", 1. - fitting_sols.get_score(i), " and rmsd to native of:", rmsd)
    IMP.atom.write_pdb(mh, "sol_" + str(i) + ".pdb")
    IMP.core.transform(
        prot_rb, fitting_sols.get_transformation(i).get_inverse())
print("done")



from __future__ import print_function
import IMP
import subprocess
import random
import IMP.domino
import IMP.core
import IMP.rmf
import RMF

import time
import IMP.algebra
import types
import re
import sys
import operator
import os
from . import atomicDominoUtilities


class PeptideDocker:

    def __init__(self, parameterFileName):
        self.createdMdOptimizer = 0
        self.fixedNonFlexibleAtoms = 0
        self.readParameters(parameterFileName)
        self.wroteNativeProtein = 0

    def loadHelpers(self):
        self.loadDockingHelpers()
        self.loadMdHelpers()

    def readParameters(self, parameterFileName):
        self.parameters = atomicDominoUtilities.readParameters(
            parameterFileName)

    # Processing specific to peptide docking and this script
    def loadDockingHelpers(self):
        outputDir = self.getParam("output_directory")
        mkdirProcess = subprocess.Popen(
            ['mkdir',
             '-p',
             self.getParam('output_directory')],
            shell=False,
            stderr=subprocess.PIPE)
        output = mkdirProcess.communicate()

        self.namesToParticles = atomicDominoUtilities.makeNamesToParticles(
            self.protein)

        self.times = []
        self.rawTimes = []

    # Helper method; create dictionary where the keys are names of all
    # particles in the system and the values are themselves dictionaries
    def initializeParticleNameDict(self, leaves):
        particleDict = {}

        for leaf in leaves:
            name = self.quickParticleName(leaf)
            particleDict[name] = {}
        return particleDict

    def getModel(self):
        return self.model

    def getProtein(self):
        return self.protein

    def createModel(self):

        pdbName = self.getParam("native_pdb_input_file")

        self.model = IMP.Model()

        self.protein = IMP.atom.read_pdb(
            pdbName,
            self.model,
            IMP.atom.ATOMPDBSelector())

        # atom radii, forcefield topology, etc. -- no restraints created here

        ff = IMP.atom.CHARMMParameters(IMP.atom.get_data_path("top_heav.lib"),
                                       IMP.atom.get_data_path("par.lib"))

        topology = ff.create_topology(self.protein)
        topology.apply_default_patches()

        topology.add_atom_types(self.protein)

        ff.add_radii(self.protein)
        ff.add_well_depths(self.protein)
        self.ff = ff
        self.topology = topology

    # Get parameter value. Throws dictionary exception if not found
    def getParam(self, paramName):
        paramValue = self.parameters[paramName]
        return paramValue

    # Peptide-docking friendly way to log which restraint we're dealing with
    def quickRestraintName(self, r):
        nameList = []
        ps = r.get_input_particles()

        for p in ps:
            if (self.isAtomParticle(p) == 0):
                continue
            isFlexible = 0
            if ((self.quickParticleName(p) in self.flexibleAtoms) == 1):
                isFlexible = 1
            nameList.append("%s_%s" % (self.quickParticleName(p), isFlexible))

        return "%s acting on %s" % (r.get_name(), " , ".join(nameList))

    def quickParticleName(self, particle):
        return atomicDominoUtilities.quickParticleName(particle)

    # output current time
    def logTime(self, label):
        nextTime = time.asctime(time.localtime(time.time()))
        print("%s:\t %s" % (label, nextTime))
        nextTimeEntry = [label, nextTime]
        self.times.append(nextTimeEntry)
        self.rawTimes.append(time.time())

    # output all times that were saved by logTime()
    def outputTimes(self):
        print("Time for each step:")
        for timeEntry in self.times:
            label = timeEntry[0]
            value = timeEntry[1]
            print("%s:\t%s" % (label, value))

    def getFlexibleAtoms(self):
        return self.flexibleAtoms

    # Helper method to quickly get the chain identifier for a particle
    def getChain(self, particle):
        atomDecorator = IMP.atom.Atom(particle)
        residue = IMP.atom.get_residue(atomDecorator)
        chain = IMP.atom.get_chain(residue)
        chainId = chain.get_id()
        return chainId

    # Helper method to quickly get the residue index for a particle
    def getResidue(self, particle):
        atomDecorator = IMP.atom.Atom(particle)
        residue = IMP.atom.get_residue(atomDecorator)
        residueNumber = residue.get_index()
        return residueNumber

    def isAtomParticle(self, p):
        return atomicDominoUtilities.isAtomParticle(p)

    # Read all particles in restraint; return 1 if all are flexible and 0 if
    # at least one atom is fixed
    def allFlexibleAtoms(self, r):
        ps = r.get_input_particles()
        for p in ps:
            if (self.isAtomParticle(p) == 0):
                continue
            if ((self.quickParticleName(p) in self.flexibleAtoms) == 0):
                return 0
        return 1

    # Check if we should skip processing of this pair of particles. Don't add restraints between particles that aren't optimized,
    # and don't add restraints between particles on the same residue (for now
    # assume stereochemistry is good enough for this)
    def skipNonBondedPair(self, pair):

        # skip if same residue
        if (self.getResidue(pair[0]) == self.getResidue(pair[1])):
            return 1

        # skip if neither is flexible
        if ((self.quickParticleName(pair[0]) in self.flexibleAtoms) == 0 and (self.quickParticleName(pair[1]) in self.flexibleAtoms) == 0):
            return 1
        return 0

    # Set up degrees of freedom for the system. Either we are dealing with a fixed protein or a flexible one.
    # Determine which atoms are flexible and save them in self.flexibleAtoms
    def initDof(self):

        peptideChain = self.getParam("peptide_chain")
        self.flexibleAtoms = {}

        if (self.getParam("flexible_protein") == "yes"):
            # All atoms within range of the peptide are flexible; range defined
            # by close_pair_distance param
            particlePairs = self.getClosePairs()

            for pair in particlePairs:
                p0 = pair[0]
                p1 = pair[1]
                p0Chain = self.getChain(p0)
                p1Chain = self.getChain(p1)
                if (p0Chain == peptideChain or p1Chain == peptideChain):
                    self.flexibleAtoms[self.quickParticleName(p0)] = 1
                    self.flexibleAtoms[self.quickParticleName(p1)] = 1

        else:
            # only peptide atoms are flexible, determined by the chain id
            leaves = IMP.atom.get_leaves(self.protein)
            for leaf in leaves:
                if (self.getChain(leaf) == peptideChain):
                    self.flexibleAtoms[self.quickParticleName(leaf)] = 1
        self.fixNonFlexibleAtoms()

    # return a dictionary where the keys are the names of the fixed atoms in
    # the system
    def getFixedAtoms(self):
        leaves = IMP.atom.get_leaves(self.protein)
        fixedAtoms = {}
        for leaf in leaves:
            leafName = self.quickParticleName(leaf)
            if (leafName in self.flexibleAtoms) == 0:
                fixedAtoms[leafName] = 1
        return fixedAtoms

    # for each flexible atom, give it the coordinates of another flexible atom (ensures that each atom starts out in the binding site)
    # Although it does guarantee a very random initial configuration, it is
    # cheating since we are using knowledge of the native state
    def initializeRandomExisting(self):
        leaves = IMP.atom.get_leaves(self.protein)
        particlesToVary = []
        xyzs = []
        for leaf in leaves:
            if ((self.quickParticleName(leaf) in self.flexibleAtoms) == 1):
                particlesToVary.append(self.quickParticleName(leaf))
        atomToCoordinates = {}
        for pName in particlesToVary:
            randomPname = random.choice(particlesToVary)
            randomP = self.namesToParticles[randomPname]
            print("setting coordinates for particle %s to the ones currently set for particle %s" % (pName, randomPname))
            atomToCoordinates[pName] = IMP.core.XYZ(randomP).get_coordinates()

        for pName in particlesToVary:
            particle = self.namesToParticles[pName]
            xyz = IMP.core.XYZ(particle)
            xyz.set_coordinates(atomToCoordinates[pName])
            xyzs.append(xyz)
        return xyzs

    # Set initial positions of flexible atoms in the system. We are
    # experimenting with the best way to do this
    def setInitialPositions(self):
        initialPositionMethod = self.getParam("initial_position_method")

        # set random coordinates for atom by creating bounding box around native coordinates, increasing it
        # by defined offset plus random number (necessary when running on
        # cluster)
        if (initialPositionMethod == "random"):

            myRandom = random.random()
            initialOffset = float(
                self.getParam("initial_atom_offset")) + myRandom
            for particleName in self.flexibleAtoms.keys():
                p = self.namesToParticles[particleName]
                xyzDecorator = IMP.core.XYZ(p)
                coordinates = xyzDecorator.get_coordinates()
                bb = IMP.algebra.BoundingBox3D(coordinates)
                bb += initialOffset
                bb += myRandom

                randomXyz = IMP.algebra.get_random_vector_in(bb)
                xyzDecorator.set_coordinates(randomXyz)

        # for each flexible atom, give it the coordinates of another flexible
        # atom (ensures that each atom starts out in the binding site)
        elif (initialPositionMethod == "random_existing"):
            print("start random existing")
            xyzs = self.initializeRandomExisting()

        # same as random_existing except offset each atom a little (doesn't
        # seem to make too much of a difference)
        elif (initialPositionMethod == "random_existing_box"):
            xyzs = self.initializeRandomExisting()
            initialOffset = float(self.getParam("initial_atom_offset"))
            for xyzDecorator in xyzs:
                coordinates = xyzDecorator.get_coordinates()
                bb = IMP.algebra.BoundingBox3D(coordinates)
                bb += initialOffset

                randomXyz = IMP.algebra.get_random_vector_in(bb)
                xyzDecorator.set_coordinates(randomXyz)

        elif (initialPositionMethod == "random_box"):

            # Get bounding box containing all particles within close pair distance of flexible atoms
            # This doesn't work perfectly; can put initial positions in the fixed atoms accidentally
            # As constructed it is specific to peptide atoms only and not
            # flexible protein atoms
            bsParticles = []

            for firstPname in self.nonBondedPairs.keys():
                secondPnames = self.nonBondedPairs[firstPname]
                for secondPname in secondPnames.keys():
                    if ((secondPname in self.flexibleAtoms) == 0):
                        fixedAtom = self.namesToParticles[firstPname]

                        bsParticles.append(IMP.core.XYZ(fixedAtom))
                        print("added next close atom %s to bounding box particles" % firstPname)
            # bsBoundingBox = IMP.core.get_bounding_box(bsParticles)  -- revert
            # if I can figure out how
            peptideParticles = []
            leaves = IMP.atom.get_leaves(self.protein)
            for leaf in leaves:
                if (self.getChain(leaf) == self.getParam("peptide_chain")):
                    peptideParticles.append(IMP.core.XYZ(leaf))
            bsBoundingBox = IMP.core.get_bounding_box(peptideParticles)
            # set initial position for each flexible atoms, and adjust it if it
            # is too close to a fixed residue
            fixedAtoms = self.getFixedAtoms()
            for flexPname in self.flexibleAtoms.keys():
                goodPosition = 0
                flexParticle = self.namesToParticles[flexPname]
                flexXyzDecorator = IMP.core.XYZ(flexParticle)
                print("processing position for pname %s" % flexPname)
                while(goodPosition == 0):
                    goodPosition = 1
                    flexXyzDecorator.set_coordinates(
                        IMP.algebra.get_random_vector_in(bsBoundingBox))
                    for fixedPname in fixedAtoms.keys():
                        fixedParticle = self.namesToParticles[fixedPname]
                        distance = IMP.algebra.get_distance(
                            IMP.core.XYZ(fixedParticle).get_coordinates(),
                            IMP.core.XYZ(flexParticle).get_coordinates())
                        if (distance < 2):
                            print("finding new position for %s" % flexPname)
                            goodPosition = 0
                            # start the while loop over which will set new
                            # coordinates
                            break

        # take a box around the whole protein and randomly initialize atoms
        # here
        elif (initialPositionMethod == "random_full"):
            bb = IMP.atom.get_bounding_box(self.protein)
            for particleName in self.flexibleAtoms.keys():
                print("creating random position for particle %s" % particleName)
                p = self.namesToParticles[particleName]
                xyzDecorator = IMP.core.XYZ(p)
                randomXyz = IMP.algebra.get_random_vector_in(bb)
                myRandom = random.random()
                bb += myRandom
                xyzDecorator.set_coordinates(randomXyz)

        # Read in a file generated from a previous run (fastest way is to
        # create a new model and copy corresponding XYZ Decorators)
        elif (initialPositionMethod == "file"):

            initialPositionFile = self.getParam("saved_initial_atom_positions")
            print("reading initial positions from %s" % initialPositionFile)
            initialModel = IMP.Model()
            initialProtein = IMP.atom.read_pdb(
                initialPositionFile,
                initialModel,
                IMP.atom.ATOMPDBSelector())
            initialLeaves = IMP.atom.get_leaves(initialProtein)
            for initialLeaf in initialLeaves:
                name = self.quickParticleName(initialLeaf)
                if ((name in self.namesToParticles) == 1):
                    existingLeaf = self.namesToParticles[name]
                    existingLeafXyz = IMP.core.XYZ(existingLeaf)
                    initialLeafXyz = IMP.core.XYZ(initialLeaf)
                    existingLeafXyz.set_coordinates(
                        initialLeafXyz.get_coordinates())
                else:
                    print("Read in initial positions from file %s but this file contained particle %s which is not in current model" % (initialPositionFile, name))
        else:
            print("Please specify valid initial position method (random or file)\n")
            sys.exit()

        # output initial positions
        outputDir = self.getParam("output_directory")
        initialAtomPositionFile = self.getParam(
            "initial_atom_positions_output_file")
        fullOutputFile = os.path.join(outputDir, initialAtomPositionFile)
        print("writing output to file %s" % fullOutputFile)
        IMP.atom.write_pdb(self.protein, fullOutputFile)

        # get initial score for model
        initialScore = self.model.evaluate(False)
        print("initial score for model is %s" % initialScore)

    # Get non-bonded pairs in the system in preparation for restraining them
    def getNonBondedPairs(self):
        nonBondedDefinition = self.getParam("non_bonded_definition")
        particlePairs = []
        if (nonBondedDefinition == "close_pairs"):
            particlePairs = self.getClosePairs()
        elif (nonBondedDefinition == "random"):
            particlePairs = self.getRandomParticlePairs()
        elif (nonBondedDefinition == "manual"):
            particlePairs = self.readManualRestraints(nativeProtein, 6)
        else:
            print("Please specify valid non bonded pair definition")
            sys.exit()
        return particlePairs

    # Get 3D distance between the particles in the pair
    def getXyzDistance(self, pair):
        d0 = IMP.core.XYZ(pair[0])
        d1 = IMP.core.XYZ(pair[1])
        distance = IMP.core.get_distance(d0, d1)
        return distance

    # Get all pairs of particles within the distance defined by
    # close_pair_distance parameter
    def getClosePairs(self):

        # Get a list of all atoms in the protein, and put it in a container
        leaves = IMP.atom.get_leaves(self.protein)
        cont = IMP.container.ListSingletonContainer(leaves)
        closePairDistance = float(self.getParam("close_pair_distance"))
        # sometimes closePairContainer returns particles > distanceCutoff, even
        # with slack = 0
        useHardCutoff = self.getParam("use_hard_distance_cutoff")

        nbl = IMP.container.ClosePairContainer(cont, closePairDistance, 0.0)

        self.model.evaluate(False)

        particlePairs = nbl.get_particle_pairs()
        finalPairs = []
        for pair in particlePairs:
            distance = self.getXyzDistance(pair)
            q0 = self.quickParticleName(pair[0])
            q1 = self.quickParticleName(pair[1])

            if (useHardCutoff == "0" or (useHardCutoff == "1" and distance < closePairDistance)):
                finalPairs.append([pair[0], pair[1]])

        return finalPairs

    # Add restraints for bond lengths, angles, dihedrals, and impropers
    def addForceFieldRestraints(self):

        useForcefieldRestraints = self.getParam("use_forcefield_restraints")
        if (useForcefieldRestraints == "1"):

            ff = IMP.atom.CHARMMParameters(
                IMP.atom.get_data_path("top_heav.lib"),
                IMP.atom.get_data_path("par.lib"))

            # Set up and evaluate the stereochemical part (bonds, angles, dihedrals,
            # impropers) of the CHARMM forcefield
            bonds = self.topology.add_bonds(self.protein)
            angles = ff.create_angles(bonds)
            dihedrals = ff.create_dihedrals(bonds)
            impropers = self.topology.add_impropers(self.protein)

            # tracks which particles are together in forcefield restraints
            # (useful for filtering excluded volume)
            self.ffParticleGroups = self.initializeParticleNameDict(
                IMP.atom.get_leaves(self.protein))

            self.createForceFieldRestraintsForModel(
                bonds,
                "bonds",
                "Bond Restraints",
                IMP.atom.BondSingletonScore(
                    IMP.core.Harmonic(
                        0,
                        1)))
            self.createForceFieldRestraintsForModel(
                angles,
                "angles",
                "Angle Restraints",
                IMP.atom.AngleSingletonScore(
                    IMP.core.Harmonic(
                        0,
                        1)))
            self.createForceFieldRestraintsForModel(
                dihedrals,
                "dihedrals",
                "Dihedral Restraints",
                IMP.atom.DihedralSingletonScore())
            self.createForceFieldRestraintsForModel(
                impropers,
                "impropers",
                "Improper Restraints",
                IMP.atom.ImproperSingletonScore(
                    IMP.core.Harmonic(
                        0,
                        1)))

        else:
            print ("Skipping forcefield restraints")

    # Create one type of forcefield restraint. The restraint is initially across the full model and needs to be decomposed
    # so it can be used in the restraint graph.
    def createForceFieldRestraintsForModel(
        self,
        ffParticles,
        containerName,
        restraintName,
            singletonScore):

        maxForceFieldScore = float(self.getParam("max_forcefield_score"))
        cont = IMP.container.ListSingletonContainer(self.model, containerName)
        cont.add_particles(ffParticles)
        listBondRestraint = IMP.container.SingletonsRestraint(
            singletonScore, cont, restraintName)

        # Restraint decomposition has been a little touchy; a couple
        # workarounds contained
        decomposedRestraintTemp = listBondRestraint.create_decomposition()
        decomposedRestraints = IMP.RestraintSet.get_from(
            decomposedRestraintTemp)
        rs_restraints = decomposedRestraints.get_restraints()

        # manually remove the full restraint after getting the decomposition
        del listBondRestraint

        count = decomposedRestraints.get_number_of_restraints()
        for i in range(count):

            r = decomposedRestraints.get_restraint(i)

            if (self.allFlexibleAtoms(r) == 1):
                # add the decomposed restraint to the model
                self.model.add_restraint(r)
                r.set_maximum_score(maxForceFieldScore)
                self.addParticlesToGroups(r)
            else:
                t = 1
                # self.model.remove_restraint(r)

    # Add all atoms in r to ffParticleGroups; tracks which atoms are restrained by forcefield terms so
    # we don't restrain them in other methods (e.g. excluded volume) --
    # essentially a manual pair-filter
    def addParticlesToGroups(self, r):
        inputParticles = []
        ps = r.get_input_particles()

        for p in ps:

            if (self.isAtomParticle(p) == 0):  # skip particles that aren't actually atoms (e.g. bond particles)
                continue
            inputParticles.append(p.get_name())

        for firstName in inputParticles:
            for secondName in inputParticles:
                if (firstName == secondName):
                    continue
                self.ffParticleGroups[firstName][secondName] = 1

    # Add excluded volume restraints across close pairs. Still a work in progress as they have been slowing down
    # the code quite a bit and we'll probably want to turn them on only at a
    # defined time
    def addExcludedVolumeRestraints(self):

        restrainedParticles = atomicDominoUtilities.getRestrainedParticles(
            self.protein, self.model, self.namesToParticles)
        lsc = IMP.container.ListSingletonContainer(restrainedParticles)

        evr = IMP.core.ExcludedVolumeRestraint(lsc, 1, 1)
        self.model.add_restraint(evr)

    # Add lennard jones restraints between flexible atoms if they haven't been
    # added already, and then scale radii of flexible atoms by input scaling
    # factor
    def addFlexFlexLjRestraints(self, scalingFactor):
        if (self.initializedFlexFlexLjRestraints == 0):
            print("adding initial lennard jones restraints between pairs of flexible atoms")
            self.initializedFlexFlexLjRestraints = 1
            leaves = IMP.atom.get_leaves(self.protein)
            counter = 0

            sf = IMP.atom.ForceSwitch(6.0, 7.0)
            ps = IMP.atom.LennardJonesPairScore(sf)
            for i in range(len(leaves)):
                iLeaf = leaves[i]
                for j in range(i + 1, len(leaves)):
                    jLeaf = leaves[j]
                    if ((self.quickParticleName(iLeaf) in self.flexibleAtoms) == 1 and (self.quickParticleName(jLeaf) in self.flexibleAtoms) == 1):
                        ljpr = IMP.core.PairRestraint(
                            ps, [iLeaf, jLeaf], "LennardJones %s_%s" %
                            (i, j))
                        self.model.add_restraint(ljpr)
        chainHierarchies = IMP.atom.get_by_type(
            self.protein, IMP.atom.CHAIN_TYPE)
        peptideChainHierarchy = None
        print("scaling atomic radii by scaling factor %s" % scalingFactor)
        # reset radii back to original in preparation for rescaling
        self.ff.add_radii(self.protein)
        for pName in self.flexibleAtoms.keys():
            particle = self.namesToParticles[pName]
            xyzr = IMP.core.XYZR(particle)
            radius = xyzr.get_radius()
            radius *= scalingFactor
            xyzr.set_radius(radius)

    # add lennard-jones restraints between all fixed atoms and all flexible
    # atoms
    def addFixedFlexibleLjRestraints(self):
        print("adding initial fixed flexible lj restraints")
        fixedAtoms = self.getFixedAtoms()
        sf = IMP.atom.ForceSwitch(6.0, 7.0)
        ps = IMP.atom.LennardJonesPairScore(sf)

        for flexPname in self.flexibleAtoms.keys():
            flexParticle = self.namesToParticles[flexPname]
            for fixedPname in fixedAtoms.keys():
                fixedParticle = self.namesToParticles[fixedPname]
                ljpr = IMP.core.PairRestraint(
                    ps, [fixedParticle, flexParticle], "LennardJones %s_%s" %
                    (flexPname, fixedPname))
                self.model.add_restraint(ljpr)

    # actually do the work to create the DOPE restraint
    def createDopeRestraint(self, pair):
        q0 = self.quickParticleName(pair[0])
        q1 = self.quickParticleName(pair[1])
        # create restraint around this distance, add to model
        prName = "NonBonded %s_%s" % (q0, q1)
        pairRestraint = IMP.core.PairRestraint(
            self.dopePairScore, pair, prName)

        # print "Added non-bonded restraint between %s and %s" % (q0, q1)
        self.model.add_restraint(pairRestraint)
        maxNonBondedScore = float(self.getParam("max_non_bonded_score"))
        pairRestraint.set_maximum_score(maxNonBondedScore)
        #initialScore = pairRestraint.evaluate(0)

    # add non-bonded restraints across close pairs. At least one atom in each
    # pair should be flexible
    def addClosePairNonBondedRestraints(self):
        particlePairs = self.getNonBondedPairs()

        nonBondedRestraintType = self.getParam("non_bonded_restraint_type")
        dopeCutoff = float(self.getParam("dope_cutoff"))

        k = IMP.core.Harmonic.get_k_from_standard_deviation(1)

        ps = 0
        if (nonBondedRestraintType == "dope"):
            IMP.atom.add_dope_score_data(self.protein)
            ps = IMP.atom.DopePairScore(dopeCutoff)
        self.dopePairScore = ps
        nonBondedCount = 0
        self.nonBondedPairs = self.initializeParticleNameDict(
            IMP.atom.get_leaves(self.protein))
        for pair in particlePairs:

            if (self.skipNonBondedPair(pair) == 1):
                continue

            self.createDopeRestraint(pair)

            nonBondedCount += 1
            self.nonBondedPairs[self.quickParticleName(pair[0])][
                self.quickParticleName(pair[0])] = 1
            self.nonBondedPairs[self.quickParticleName(pair[1])][
                self.quickParticleName(pair[1])] = 1

        print("Added %s non-bonded restraints" % nonBondedCount)
        self.model.evaluate(False)

    def addCompleteNonBondedRestraints(self):
        leaves = IMP.atom.get_leaves(self.protein)
        counter = 0

        for i in range(len(leaves)):
            iLeaf = leaves[i]
            for j in range(i + 1, len(leaves)):
                jLeaf = leaves[j]

                if ((self.quickParticleName(jLeaf) in self.nonBondedPairs[self.quickParticleName(iLeaf)]) == 1):
                    continue
                if (self.skipNonBondedPair([iLeaf, jLeaf]) == 1):
                    continue

                self.createDopeRestraint([iLeaf, jLeaf])
                counter += 1
                # print "added complete restraint between %s and %s" %
                # (self.quickParticleName(iLeaf),
                # self.quickParticleName(jLeaf))
        print("added %s restraints for %s particles" % (counter, len(leaves)))
        self.model.evaluate(False)

    # lots of output on all restraints
    def writeAllRestraints(self):
        print("write all restraints:")
        for r in IMP.get_restraints([self.model.get_root_restraint_set()]):
            print(self.quickRestraintName(r))

    # Read parameterss with md temperature / step schedule and save them
    def initializeMdSchedule(self):
        schedule = self.getParam("md_schedule")
        stages = schedule.split()
        totalSteps = 0
        for stage in stages:
            print("unpacking stage %s" % stage)
            [steps, temperature, excludedVolume] = stage.split('_')
            totalSteps += int(steps)
        self.totalMdSteps = totalSteps
        self.mdStages = stages

    # Creates roothandle for hdf5 objects interacting with md trajectory
    # Either we create a new trajectory in this file and read it back in the
    # same run, or we are reading a saved trajectory
    def createMdRootHandle(self):

        mdFileName = ""
        outputDir = self.getParam("output_directory")

        if (self.getParam("read_assignments_from_file") == "no"):
            # create new hdf5 file to which md output will be written
            mdFileName = self.getParam("md_trajectory_output_file")
            fileName = os.path.join(outputDir, mdFileName)
            print("creating rmf file with filename %s" % fileName)
            rh = RMF.create_rmf_file(fileName)
            my_kc = rh.add_category("my data")
            IMP.rmf.add_hierarchy(rh, self.protein)

            self.rootHandle = rh
        else:
            # use existing hdf5 file
            mdFileName = self.getParam("saved_md_filename")

            rh = RMF.open_rmf_file(mdFileName)
            IMP.rmf.set_hierarchies(rh, [self.protein])

            self.rootHandle = rh

    def evaluateModel(self):
        return self.model.evaluate(False)

    def loadMdHelpers(self):

        self.createMdRootHandle()

        self.createMdOptimizer()

        self.addMdOptimizerStates()

        self.initializeMdSchedule()

        self.createdMdOptimizer = 1

    # Create the optimizer object for running MD
    def createMdOptimizer(self):
        print("creating md object")
        md = IMP.atom.MolecularDynamics(self.model)
        # hack to get around bug where vx attributes in non-optimized particles
        # are not being set
        md.optimize(1)
        md.set_velocity_cap(100.0)
        self.mdOptimizer = md

    def addMdOptimizerStates(self):

        # VelocityScalingOptimizerState
        vsos = IMP.atom.VelocityScalingOptimizerState(
            IMP.atom.get_leaves(self.protein),
            0)  # temperature is set later
        vsIndex = self.mdOptimizer.add_optimizer_state(vsos)

        # SaveHierarchyConfigurationOptimizerState
        hdos = IMP.rmf.SaveHierarchyConfigurationOptimizerState(
            [self.protein], self.rootHandle)
        self.mdOptimizer.add_optimizer_state(hdos)

        self.hdos = hdos
        self.vsos = vsos

    def readTrajectories(self):

        cgFileName = self.getParam("cg_output_file")
        bestCgScore = 10000000
        bestCgScoreFile = ""
        bestCgRmsd = 10000000
        bestCgRmsdFile = ""

        outputDir = self.getParam("output_directory")
        trajectoryFile = self.getParam("md_trajectory_output_file")
        fullFile = os.path.join(outputDir, trajectoryFile)
        rh = RMF.open_rmf_file(fullFile)
        IMP.rmf.set_hierarchies(rh, [self.protein])
        framesToRead = atomicDominoUtilities.getMdIntervalFrames(
            rh, int(self.getParam("cg_interval")), self.protein)

        if (len(framesToRead) > 0):
            for cgNumber in framesToRead:
                # Open next cg trajectory
                outputDir = self.getParam("output_directory")
                fullCgFileName = os.path.join(
                    outputDir, "%s%s" %
                    (cgFileName, cgNumber))
                rh = RMF.open_rmf_file(fullCgFileName)
                IMP.rmf.set_hierarchies(rh, [self.protein])

                frameCount = IMP.rmf.get_number_of_frames(rh, self.protein)
                IMP.rmf.load_frame(rh, frameCount - 1, self.protein)
                score = self.model.evaluate(False)
                rmsd = self.calculateNativeRmsd(self.flexibleAtoms)
                print("cg number %s has score %s rmsd %s" % (cgNumber, score, rmsd))
                if (score < bestCgScore):
                    bestCgScore = score
                    bestCgScoreFile = fullCgFileName
                if (rmsd < bestCgRmsd):
                    bestCgRmsd = rmsd
                    bestCgRmsdFile = fullCgFileName

            # output best score information
            self.singlePdbResults(
                bestCgScoreFile,
                -1,
                self.getParam("best_cg_score_output_file"))
            self.singlePdbResults(
                bestCgRmsdFile,
                -1,
                self.getParam("best_cg_rmsd_output_file"))
            self.singlePdbResults(
                "%s%s" %
                (cgFileName, framesToRead[-1]), -1, self.getParam("final_cg_frame_output_file"))
            finalCgRmsd = self.calculateNativeRmsd(self.flexibleAtoms)
            self.bestCgScore = bestCgScore
            self.bestCgRmsd = bestCgRmsd
            self.finalCgRmsd = finalCgRmsd

    def singlePdbResults(self, trajectoryFile, frame, outputPdbFile):

        fullTrajectoryFile = os.path.join(
            self.getParam("output_directory"),
            trajectoryFile)
        fullOutputFile = os.path.join(
            self.getParam("output_directory"),
            outputPdbFile)
        rh = RMF.open_rmf_file(fullTrajectoryFile)
        IMP.rmf.set_hierarchies(rh, [self.protein])
        if (frame == -1):
            frame = IMP.rmf.get_number_of_frames(rh, self.protein) - 1
        IMP.rmf.load_frame(rh, frame, self.protein)
        IMP.atom.write_pdb(self.protein, fullOutputFile)

    def calculateRmsd(self, otherProtein, flexibleAtoms):
        otherNamesToParticles = atomicDominoUtilities.makeNamesToParticles(
            otherProtein)
        otherVector = []
        modelVector = []
        for pName in otherNamesToParticles.keys():
            if ((pName in flexibleAtoms) == 0):
                continue
            otherParticle = otherNamesToParticles[pName]
            modelParticle = self.namesToParticles[pName]
            otherVector.append(
                IMP.core.XYZ(otherParticle).get_coordinates())
            modelVector.append(
                IMP.core.XYZ(modelParticle).get_coordinates())
        rmsd = IMP.atom.get_rmsd(otherVector, modelVector)
        return rmsd

    def calculateNativeRmsd(self, flexibleAtoms):

        if (self.wroteNativeProtein == 0):
            pdbName = self.getParam("native_pdb_input_file")
            self.nativeModel = IMP.Model()
            self.nativeProtein = IMP.atom.read_pdb(
                pdbName,
                self.nativeModel,
                IMP.atom.ATOMPDBSelector())
            self.wroteNativeProtein = 1

        return self.calculateRmsd(self.nativeProtein, flexibleAtoms)

    # Run MD according to the temperature schedule read in from parameters
    def runMolecularDynamics(self):
        self.initializedFlexFlexLjRestraints = 0
        if (self.fixedNonFlexibleAtoms == 0):
            print("error: before running md, must fix non flexible atoms")
            sys.exit()
        stepsSoFar = 0
        for stage in self.mdStages:
            [steps, temperature, excludedVolume] = stage.split('_')
            stepsSoFar += int(steps)
            self.vsos.set_temperature(int(temperature))
            if (excludedVolume == "1"):  # add lennard jones restraints between flexible atoms and fixed atoms (don't add between flex atoms)
                self.addFixedFlexibleLjRestraints()
            elif (excludedVolume != "0"):
                # add lennard jones restraints between flexible atoms
                if (self.getParam("excluded_volume_type") == "lj"):
                    self.addFlexFlexLjRestraints(float(excludedVolume))
                elif (self.getParam("excluded_volume_type") == "ev"):
                    self.addExcludedVolumeRestraints(
                    )  # add full excluded volume restraints (just take everything in the protein)
                else:
                    "please set excluded_volume_type to either lj or ev"
                    sys.exit()
            print("running md at temperature %s for %s steps" % (temperature, steps))
            self.mdOptimizer.optimize(int(steps))

            IMP.atom.write_pdb(
                self.protein,
                os.path.join(
                    self.getParam(
                        "output_directory"),
                    "md_after_%s.pdb" %
                    stepsSoFar))
            print("model score after stage %s is %s" % (stage, self.model.evaluate(False)))
        print("done running md")

    def runAllCg(self):
        cgInterval = int(self.getParam("cg_interval"))
        outputDir = self.getParam("output_directory")
        trajectoryFile = self.getParam("md_trajectory_output_file")
        fullFile = os.path.join(outputDir, trajectoryFile)
        print("open rmf %s" % fullFile)
        rh = RMF.open_rmf_file(fullFile)
        IMP.rmf.set_hierarchies(rh, [self.protein])
        framesToRead = atomicDominoUtilities.getMdIntervalFrames(
            rh, cgInterval, self.protein)

        for i in framesToRead:
            cgFileName = "%s%s" % (self.getParam("cg_output_file"), i)
            self.applyCg(i, cgFileName)

    # Apply CG to each step of the saved MD trajectory. Writes results to same
    # file storing the MD trajectory
    def applyCg(self, mdStepStart, cgFileName):

        cgSteps = int(self.getParam("cg_steps"))

        # load md step specified by calling object
        print("apply cg: loading md frame for cg step start %s" % mdStepStart)
        IMP.rmf.load_frame(self.rootHandle, mdStepStart, self.protein)

        # create new hdf5 file to which cg output will be written
        outputDir = self.getParam("output_directory")
        fileName = os.path.join(outputDir, cgFileName)
        print("creating cg hdf5 file %s" % fileName)
        rh = RMF.create_rmf_file(fileName)
        my_kc = rh.add_category("my data")
        IMP.rmf.add_hierarchy(rh, self.protein)

        # apply cg
        cg = IMP.core.ConjugateGradients(self.model)
        firstScore = self.model.evaluate(False)

        print("creating optimizer state")
        hdos = IMP.rmf.SaveHierarchyConfigurationOptimizerState(
            [self.protein], rh)
        hdos.set_skip_steps(0)
        # hdos is the optimizer state writing configurations to disk
        cg.add_optimizer_state(hdos)

        print("running cg")
        cg.optimize(cgSteps)
        secondScore = self.model.evaluate(False)
        print("cg score after md step %s before %s after %s" % (mdStepStart, firstScore, secondScore))
        return secondScore

    # Tell the optimizer not to move atoms we have determined ar fixed
    def fixNonFlexibleAtoms(self):
        if (self.createdMdOptimizer == 0):
            self.createMdOptimizer()
            # print "ERROR: must create md optimizer and optimize once before set_particles_are_optimized is called"
            # sys.exit()

        self.fixedNonFlexibleAtoms = 1
        leaves = IMP.atom.get_leaves(self.protein)
        for leaf in leaves:
            if ((self.quickParticleName(leaf) in self.flexibleAtoms) == 0):
                xyzDecorator = IMP.core.XYZ(leaf)
                xyzDecorator.set_coordinates_are_optimized(0)

    def writeOutput(self):
        leaves = IMP.atom.get_leaves(self.protein)
        flexiblePeptideAtoms = 0
        flexibleProteinAtoms = 0
        fixedPeptideAtoms = 0
        fixedProteinAtoms = 0

        for leaf in leaves:
            if ((self.quickParticleName(leaf) in self.flexibleAtoms) == 0):
                if (self.getChain(leaf) == self.getParam("peptide_chain")):
                    fixedPeptideAtoms += 1
                else:
                    fixedProteinAtoms += 1
            else:
                if (self.getChain(leaf) == self.getParam("peptide_chain")):
                    flexiblePeptideAtoms += 1
                else:
                    flexibleProteinAtoms += 1
        print("Flexible peptide atoms: %s" % flexiblePeptideAtoms)
        print("Fixed peptide atoms: %s" % fixedPeptideAtoms)
        print("Flexible protein atoms: %s" % flexibleProteinAtoms)
        print("Fixed protein atoms: %s" % fixedProteinAtoms)

    def writeOsOutput(self):

        print("Best cg score: %s" % self.bestCgScore)
        print("best cg rmsd: %s" % self.bestCgRmsd)
        print("final cg rmsd: %s" % self.finalCgRmsd)



## \example em/analyze_convergence.py
# Analyze the convergence of the IMP.em.FitRestraint. The script builds a
# simple model and then displays the derivatives, em score and how well
# conjugate gradients converges under various displacements of the model.

from __future__ import print_function
import IMP.display
import IMP.em
import sys

IMP.setup_from_argv(sys.argv, "analyze convergence")

use_rigid_bodies = True
bd = 10
radius = 10

m = IMP.Model()
p = IMP.Particle(m)
IMP.atom.Mass.setup_particle(p, 10000)
d = IMP.core.XYZR.setup_particle(p)
d.set_radius(radius)

# Set up the particle as either a rigid body or a simple ball
if use_rigid_bodies:
    prb = IMP.Particle(m)
    prb.set_name("rigid body")
    d.set_coordinates(IMP.algebra.Vector3D(0, 0, 0))
    drb = IMP.core.RigidBody.setup_particle(
        prb, IMP.algebra.ReferenceFrame3D())
    drb.add_member(p)
    print("initial frame", drb.get_reference_frame())
    fp = prb
    drb.set_coordinates_are_optimized(True)
    to_move = drb
    fp = d
else:
    fp = d
    to_move = d
    d.set_coordinates_are_optimized(True)


bb = IMP.algebra.BoundingBox3D(
    IMP.algebra.Vector3D(-bd - radius, -bd - radius, -bd - radius),
    IMP.algebra.Vector3D(bd + radius, bd + radius, bd + radius))

dheader = IMP.em.create_density_header(bb, 1)
dheader.set_resolution(1)
dmap = IMP.em.SampledDensityMap(dheader)
dmap.set_particles([p])

dmap.resample()
# computes statistic stuff about the map and insert it in the header
dmap.calcRMS()
IMP.em.write_map(dmap, "map.mrc", IMP.em.MRCReaderWriter())
rs = IMP.RestraintSet(m)
# rs.set_weight(.003)

# if rigid bodies are used, we need to define a refiner as
# FitRestraint doesn't support just passing all the geometry
r = IMP.em.FitRestraint([fp], dmap)
sf = IMP.core.RestraintsScoringFunction([rs, r])
g = IMP.core.XYZDerivativeGeometry(d)
g.set_name("deriv")
w = IMP.display.PymolWriter("derivatives.pym")
# kind of abusive
steps = 4
m.set_log_level(IMP.SILENT)

opt = IMP.core.ConjugateGradients(m)
opt.set_scoring_function(sf)

def try_point(i, j, k):
    print("trying", i, j, k)
    vc = IMP.algebra.Vector3D(i, j, k)
    to_move.set_coordinates(vc)
    # display the score at this position
    cg = IMP.display.SphereGeometry(IMP.algebra.Sphere3D(vc, 1))
    cg.set_name("score")
    v = sf.evaluate(True)
    cg.set_color(IMP.display.get_hot_color(v))
    w.add_geometry(cg)
    print("score and derivatives", v, to_move.get_derivatives())
    w.add_geometry(g)

    opt.optimize(10)
    print("after", d.get_coordinates())
    mag = to_move.get_coordinates().get_magnitude()

    converge_color = IMP.display.get_gray_color(1.0 / (1.0 + mag))
    # display the distance after optimization at this position
    sg = IMP.display.SphereGeometry(IMP.algebra.Sphere3D(vc, 1))
    sg.set_color(converge_color)
    sg.set_name("converge")
    w.add_geometry(sg)

try_point(-bd, -bd, -bd)

# For a more informative (but much slower) test, use the following instead:
# for i in range(-bd, bd+1, 2*bd/steps):
#    for j in range(-bd, bd+1, 2*bd/steps):
#        for k in range(-bd, bd+1, 2*bd/steps):
#            try_point(i, j, k)



## \example pmi/multiscale.py
"""This script shows how to represent a system at multiple scales and do basic sampling.
"""

from __future__ import print_function
import IMP
import IMP.atom
import IMP.rmf
import IMP.pmi
import IMP.pmi.topology
import IMP.pmi.dof
import IMP.pmi.macros
import IMP.pmi.restraints
import IMP.pmi.restraints.stereochemistry
import sys

IMP.setup_from_argv(sys.argv, "Representation at multiple scales")
if IMP.get_is_quick_test():
    print("This example is too slow to test in debug mode - run without")
    print("internal tests enabled, or without the --run-quick-test flag")
    sys.exit(0)

###################### SYSTEM SETUP #####################
# Read sequences etc
seqs = IMP.pmi.topology.Sequences(IMP.pmi.get_example_path('data/1WCM.fasta'))
components = ["Rpb1","Rpb2","Rpb3","Rpb4"]
colors = ['medium purple','goldenrod','orchid','olive drab']
chains = "ABCD"
beadsize = 10

# Setup System and add a State
mdl = IMP.Model()
s = IMP.pmi.topology.System(mdl)
st = s.create_state()

# Add Molecules for each component as well as representations
mols = []
for n in range(len(components)):
    print('PMI: setting up',components[n],'1WCM:'+chains[n])
    mol = st.create_molecule(                      # create molecule
        components[n],
        sequence=seqs['1WCM:'+chains[n]],
        chain_id=chains[n])
    atomic = mol.add_structure(IMP.pmi.get_example_path('data/1WCM_fitted.pdb'),
                               chain_id=chains[n],
                               offset=0)
    mol.add_representation(atomic,                 # res 1,10 for structured regions
                           resolutions=[1,10],
                           color=colors[n])
    mol.add_representation(mol[:]-atomic,          # res 10 for unstructured regions
                           resolutions=[beadsize],
                           color=colors[n])
    mols.append(mol)

# calling System.build() creates all States and Molecules (and their representations)
#  Once you call build(), anything without representation is destroyed.
#  You can still use handles like molecule[a:b], molecule.get_atomic_residues() or molecule.get_non_atomic_residues()
#  However these functions will only return BUILT representations
root_hier = s.build()

# Uncomment this for verbose output of the representation
#IMP.atom.show_with_representations(root_hier)

# Setup degrees of freedom
#  The DOF functions automatically select all resolutions
#  Objects passed to nonrigid_parts move with the frame but also have their own independent movers.
dof = IMP.pmi.dof.DegreesOfFreedom(mdl)
for mol in mols:
    dof.create_rigid_body(mol,
                          nonrigid_parts=mol.get_non_atomic_residues(),
                          max_trans=0.1,
                          max_rot=0.78,
                          nonrigid_max_trans=0.1)
    # display the bonds between consecutive fragments,
    # so that they are shown in the psf
    IMP.pmi.tools.display_bonds(mol)


###################### RESTRAINTS #####################
output_objects = [] # keep a list of functions that need to be reported

# Connectivity keeps things connected along the backbone (ignores if inside same rigid body)
crs = []
for mol in mols:
    cr = IMP.pmi.restraints.stereochemistry.ConnectivityRestraint(mol)
    cr.add_to_model()
    output_objects.append(cr)
    crs.append(cr)

# Excluded volume - automatically more efficient due to rigid bodies
evr = IMP.pmi.restraints.stereochemistry.ExcludedVolumeSphere(included_objects = mols)
evr.add_to_model()
output_objects.append(evr)


###################### SAMPLING #####################
# First shuffle the system
IMP.pmi.tools.shuffle_configuration(root_hier,
                                    max_translation=30)

# Quickly move all flexible beads into place
dof.optimize_flexible_beads(100)

# Run replica exchange Monte Carlo sampling
rex=IMP.pmi.macros.ReplicaExchange0(mdl,
                                    root_hier=root_hier,                          # pass the root hierarchy
                                    crosslink_restraints=crs,                     # will display like XLs
                                    monte_carlo_sample_objects=dof.get_movers(),  # pass MC movers
                                    global_output_directory='multiscale_output/',
                                    output_objects=output_objects,
                                    monte_carlo_steps=10,
                                    number_of_best_scoring_models=0,      # set >0 to store best PDB files (but this is slow to do online)
                                    number_of_frames=1)                   # increase number of frames to get better results!
rex.execute_macro()



## \example domino/save_assignments.py
# It can often be useful to save the assignments to a file. \imp provides
# support to do this to a data set in an HDF5 file.
#
# You can use \c h5dump to view the contents of the created file.

import IMP.domino
import sys

IMP.setup_from_argv(sys.argv, "save assignments")

# create a model and some particles; they are just used as markers here
m = IMP.Model()
ps = [IMP.Particle(m) for i in range(0, 10)]

# create a subset with a few of the particles
ss = IMP.domino.Subset([ps[3], ps[5], ps[7]])

file_name = IMP.create_temporary_file_name("assignments", ".asn")

print("File name is", file_name)

# create a list of assignments

IMP.set_log_level(IMP.MEMORY)
asl = IMP.domino.WriteAssignmentContainer(file_name, ss, ps, "writer")
written = []
for i in range(0, 5):
    for j in range(0, 5):
        for k in range(0, 5):
            a = IMP.domino.Assignment([i, j, k])
            written.append(a)
            asl.add_assignment(a)

del asl

# to check, we can read it back immediately
back_asl = IMP.domino.ReadAssignmentContainer(file_name, ss, ps, "reader")


if back_asl.get_assignments() == written:
    print("They match!")
else:
    print(back_asl.get_assignments())
    print(written)
    raise RuntimeError("They don't match :-(")


# More interestingly, we can create a new model and read back the
# assignments for that
mp = IMP.Model()
psp = [IMP.Particle(mp) for i in range(0, 10)]

# create a subset with a few of the particles
ssp = IMP.domino.Subset([psp[3], psp[5], psp[7]])

print("Note the subsets are differently ordered (most of the time): ", ss, ssp)

back_aslp = IMP.domino.ReadAssignmentContainer(file_name, ssp, psp, "reader2")

# however, the states are demuxed so they match the particles
print([str(a) for a in back_aslp.get_assignments()])



"""@namespace IMP.EMageFit.imp_general.alignments
   Utility functions to handle alignments.
"""

import IMP
import IMP.atom
import IMP.algebra
import sys
import logging

log = logging.getLogger("alignments")


def get_reference_frames_from_chain_alignment(reference_rbs, reference_index,
                                              rbs_to_align, index_to_align):
    """
        Align the rigid bodies rbs_to_align to the reference frames of
        reference_rbs. The rb with index_to_align is aligned to the reference rb
        with reference_index. The function returns the reference frames to
        apply to the rbs_to_align.
    """
    ref_coords = \
        [m.get_coordinates()
         for m in reference_rbs[reference_index].get_members()]
    coords = [m.get_coordinates()
              for m in rbs_to_align[index_to_align].get_members()]
    if(len(coords) != len(ref_coords)):
        raise ValueError(
            "Mismatch in the number of members. Reference %d Aligned %d " % (
                len(ref_coords), len(coords)))
    T = IMP.algebra.get_transformation_aligning_first_to_second(coords,
                                                                ref_coords)
    new_refs = []
    for rb in rbs_to_align:
#        log.debug("aligning ... %s",rb)
        t = rb.get_reference_frame().get_transformation_to()
        new_t = IMP.algebra.compose(T, t)
        new_refs.append(IMP.algebra.ReferenceFrame3D(new_t))
    return new_refs


def align_centroids_using_pca(ref_frames, ref_frames_reference):
    """
        Align the centroids of 2 sets of rigid bodies using PCA using their
        reference frames. returns the best rmsd and the ref_frames to get it.
    """
    if(len(ref_frames) != len(ref_frames_reference)):
        raise ValueError("The number of reference frames must be the same")
    Ts1 = [r.get_transformation_to() for r in ref_frames]
    vs1 = [T.get_translation() for T in Ts1]
    Ts2 = [r.get_transformation_to() for r in ref_frames_reference]
    vs2 = [T.get_translation() for T in Ts2]

    # align with PCA
    pc1 = IMP.algebra.get_principal_components(vs1)
    pc2 = IMP.algebra.get_principal_components(vs2)
    pcTs = IMP.algebra.get_alignments_from_first_to_second(pc1, pc2)
    best_refs = []
    best_rmsd = 1e5
    for j, pcT in enumerate(pcTs):
        new_Ts1 = [IMP.algebra.compose(pcT, T) for T in Ts1]
        new_vs1 = [T.get_translation() for T in new_Ts1]
        r = IMP.atom.get_rmsd(new_vs1, vs2)
        if(r < best_rmsd):
            best_rmsd = r
            best_refs = [IMP.algebra.ReferenceFrame3D(T) for T in new_Ts1]
    return best_rmsd, best_refs


def get_reference_frames_aligning_rbs(rbs, reference_rbs):
    """ rbs = rigid bodies """
    refs = [rb.get_reference_frame() for rb in rbs]
    ref_refs = [rb.get_reference_frame() for rb in reference_rbs]
    best_rmsd, best_refs = align_centroids_using_pca(refs, ref_refs)



#!/usr/bin/env python

"""@namespace IMP.isd.History
   Classes to store output from replicas.
"""

from __future__ import print_function
import numpy
import sys


class History:

    """Class that contains the output of one replica, used by the
    Analysis class.
    """

    def __init__(self, filename):
        self.filename = filename
        self.data = {}

    def create_category(self, name):
        self.data[name] = {}

    def create_entry(self, cat, name):
        if not cat in self.data:
            self.create_category(cat)
        self.data[cat][name] = []

    def add_data(self, cat, name, data):
        self.data[cat][name].append(data)

    def sanity_check(self):
        """checks if all entries have same length and are of constant type"""
        goodlen = None
        had_warnings = False
        for cat in self.data:
            for name in self.data[cat]:
                if goodlen is None:
                    goodlen = len(self.data[cat][name])
                if len(self.data[cat][name]) != goodlen:
                    print("category %s entry %s : length %s expected, got %s"\
                        % (cat, name, goodlen, len(self.data[cat][name])))
                    break
                goodtype = type(self.data[cat][name][0])
                for ent in self.data[cat][name]:
                    if not isinstance(ent, goodtype):
                        print("category %s entry %s : %s expected, got %s"\
                            % (cat, name, goodtype, type(ent)))
                        break

    def get_data(self, cat, name):
        return self.data[cat][name]

    def get_categories(self):
        i = sorted(self.data.keys())
        if 'step' in i:
            i.remove('step')
            i = ['step'] + i
        return i

    def get_entries(self, cat):
        i = sorted(self.data[cat].keys())
        if 'step' in i:
            i.remove('step')
            i = ['step'] + i
        return i

    def remove_NAs(self):
        for cat in self.data:
            for name in self.data[cat]:
                self.data[cat][name] = [i for i in self.data[cat][name]
                                        if i != 'N/A']

    def toc(self, out=sys.stdout):
        """print the "table of contents" of this History
        tendency is a comparison of the last 200 frames, and whether it goes up
        or down.
        mean100 is the average over the last 100 frames
        """
        if not isinstance(out, file):
            out = open(out, 'w')
        out.write(
            "category\tkey_name\tn_frames\ttype\taverage\tstd\tmean100\ttendency\n")
        for cat in self.data:
            out.write(cat + '\n')
            for name in self.data[cat]:
                ent = self.data[cat][name]
                tp = type(ent[-1])
                if tp == float:
                    tp = 'float'
                elif tp == int:
                    tp = 'int'
                if tp == 'float':
                    try:
                        ent = numpy.array(ent)
                        avg = numpy.mean(ent)
                        avg100 = numpy.mean(ent[-100:])
                        avg200 = numpy.mean(ent[-200:-100])
                        st = numpy.std(ent)
                        st200 = numpy.std(ent[-200])
                        if st200 == 0 or abs(avg200 - avg100) / st200 > 3:
                            if avg200 > avg100:
                                tend = '\\'
                            else:
                                tend = '/'
                        else:
                            tend = '--'
                        out.write(
                            "\t%20s\t%12d\t%20s\t%10f\t%10f\t%10f\t%5s\n" %
                            (name, len(ent), tp, avg, st, avg100, tend))
                        continue
                    except:
                        pass
                out.write("\t%20s\t%12d\t%20s\n" % (name, len(ent), tp))
